#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Website Design
----------------
This module includes everything required to render the public website:

- css, javascript, layout images (in design/static)
- templates for page elements and pages (in design/templates)
- the main template (design/templates/index.pt)
- required views
"""

import os
from time import time
try:
    from cStringIO import StringIO
except:
    from StringIO import StringIO
from pyramid.renderers import get_renderer, render_to_response, render
from pyramid.httpexceptions import HTTPNotFound

from nive.i18n import _
from nive.definitions import ViewModuleConf, ViewConf, ConfigurationError
from nive.definitions import IWebsite, IWebsiteRoot, IRoot, ICMSRoot, IPage, IPageElement, IObject, IFile
from nive.definitions import IViewModuleConf
from nive.views import BaseView
from nive.helper import ResolveName
    
# view module definition ------------------------------------------------------------------

#@nive_module
configuration = ViewModuleConf(
    id = "design",
    name = _(u"Website design and view"),
    static = "nive.cms.design:static",
    templates = "nive.cms.design:templates",
    mainTemplate = "index.pt",
    permission = "view",
    view = "nive.cms.design.view.Design",
    views = [
        ViewConf(id="appview",  name="",      attr="app",   context=IWebsite),
        ViewConf(id="search",   name="search",attr="search",context=IRoot),
        ViewConf(id="su",       name="su",    attr="open",  context=IRoot),
        ViewConf(id="rootview", name="",      attr="view",  context=IWebsiteRoot),
        ViewConf(id="objview",  name="",      attr="view",  context=IPage,        containment=IWebsiteRoot),
        ViewConf(id="objview",  name="",      attr="view",  context=IPageElement),
        ViewConf(id="objfile",  name="file",  attr="file",  context=IFile),
        # restricted permmissions if called in editor
        ViewConf(id="rootview", name="",      attr="view",  context=ICMSRoot, containment=IWebsite),
        ViewConf(id="objview",  name="",      attr="view",  context=IPage,    containment=ICMSRoot),
    ],
    description=__doc__
)


class Design(BaseView):
    """
    get view module as attr for view class
    """
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        # the viewModule is used for template/template directory lookup 
        self.viewModule = context.app.QueryConfByName(IViewModuleConf, "design")
        if not self.viewModule:
            raise ConfigurationError, "'design' view module configuration not found"
        #self.viewModule = configuration
        self.appRequestKeys = []
        self._t = time()
        self.fileExpires = 3600
        
    @property
    def editorview(self):
        """
        Tries to load the editor view class. If none is registered the function 
        will simply return None. Otherwise the editor view class instance with 
        context and request set.
        """
        if hasattr(self, "_c_editor"):
            return self._c_editor
        # restrict editor to roots with ICMSRoot Interface. Otherwise the editor views
        # will not be found 
        root = self.context.root()
        if not ICMSRoot.providedBy(root):
            return None
        module = self.context.app.QueryConfByName(IViewModuleConf, "editor")
        if not module:
            return None
        cls =  ResolveName(module.view)
        editor = cls(self.context, self.request)
        self._c_editor = editor
        return editor
        
    # routed views -------------------------------------------------------------------------------------
    
    def view(self):
        # redirect if page is linked
        if IPage.providedBy(self) and self.context.IsLinked():
            return self.Redirect(self.context.data["pagelink"]) 
        values = {u"cmsview": self.editorview, u"context": self.context, u"view": self} 
        return self.DefaultTemplateRenderer(values)
    
    def search(self):
        values = {u"cmsview": self.editorview, u"context": self.context, u"view": self}
        name = u"search.pt"
        return self.DefaultTemplateRenderer(values, templatename=name)

   # main template ------------------------------------------------------------
        
    def index_tmpl(self):
        tmpl = self._LookupTemplate(self.viewModule.mainTemplate)
        i = get_renderer(tmpl).implementation()
        return i
       
    # redirects ----------------------------------------------------

    def app(self):
        root = self.context.GetRoot()
        url = self.PageUrl(root)
        self.Redirect(url)
    
    def open(self):
        ref = self.GetFormValue(u"r")
        page = self.context.LookupObj(ref)
        if not page:
            raise HTTPNotFound, unicode(ref)
        page = page.GetPage()
        url = self.PageUrl(page)
        self.Redirect(url)
    
        
    # html widgets ----------------------------------------------------
    
    def breadcrumbs(self, addHome=0):
        """
        """
        base = self.context.GetPage()
        parents = base.GetParents()
        parents.reverse()
        if not addHome:
            parents = parents[1:]
        if len(parents)==0 and not addHome:
            return u""
        html = StringIO()
        for page in parents:
            html.write(u"""<li><a href="%s">%s</a> <span class="divider">/</span></li>""" % (self.PageUrl(page), page.GetTitle()))
        html.write(u"""<li class="active">%s </li>""" % (base.GetTitle()))
        return html.getvalue()
    

    def navigationTop(self, addHome=1):
        """
        only first level pages
        """
        html = StringIO()
        page = self.context
        root = page.GetRoot()
        if addHome:
            highlight = u""
            if page.id == root.id:
                highlight = u"active"
            html.write(u"""<li class="%s"><a href="%s">%s</a></li>""" % (highlight, 
                                                                         self.PageUrl(root, usePageLink=1), 
                                                                         root.GetTitle()))
        
        path = page.GetParentIDs()
        pages = root.GetPages()
        for page in pages:
            if page.data.get("navHidden"):
                continue
            # highlight, current
            if page.id == self.context.id or page.id in path:
                highlight = u"active"
            else:
                highlight = u""
            # link
            html.write(u"""<li class="%s"><a href="%s">%s</a></li>""" % (highlight, 
                                                                         self.PageUrl(page, usePageLink=not self.IsEditmode()), 
                                                                         page.GetTitle()))
        return html.getvalue()


    def navigationTree(self, addHome=1, page=None, ulclass="nav nav-tabs nav-stacked"):
        """
        tree navigation for all levels
        """
        if not page:
            page = self.context.GetPage()
        root = page.GetRoot()
        html = StringIO()
        html.write(u"""<ul id="level1" class="%s">""" % (ulclass))
        if addHome:
            highlight = u""
            selected = u""
            if page.id == root.id:
                selected = u"current"
                highlight = u"active"
            html.write(u"""<li class="%s"><a href="%s" class="%s">%s</a></li>""" % (highlight, 
                                                                                    self.PageUrl(root, usePageLink=not self.IsEditmode()), 
                                                                                    selected, 
                                                                                    root.GetTitle()))
        
        path = page.GetParentIDs()
        level = 1
        html = self._navigationLevel(root, level, page, path, html, ulclass)
        html.write(u"""</ul>""")   #str(DateTime().timeTime()-t)   
        return html.getvalue()


    def navigationSub(self, page=None, ulclass="nav nav-tabs nav-stacked"):
        """
        tree subpages of current first level page
        """
        if not page:
            page = self.context.GetPage()
        base = page.GetParent()
        if page.GetID()<=0:
            return u""
        elif base.GetID()<=0:
            base = page
        else:
            while base.GetParent().GetID() > 0:
                base = base.GetParent()
        
        root = base
        path = page.GetParentIDs()
        level = 1
        html = StringIO()
        html.write(u"""<ul id="level1" class="%s">""" % (ulclass))
        html = self._navigationLevel(root, level, page, path, html, ulclass)
        html.write(u"""</ul>""")
        return html.getvalue()


    def _navigationLevel(self, current, level, active, path, io, ulclass):
        """
        """
        pages = current.GetPages()
        # sublevel
        if len(pages) and level > 1:
            io.write(u"""<li><ul id="level%d" class="%s">""" % (level, ulclass))
        
        for page in pages:
            if page.data.get("navHidden"):
                continue
        
            # highlight, current
            highlight = u""
            selected = u""
            if active.id == page.id:
                selected = u"current"
                highlight = u"active"
            elif page.id in path:
                highlight = u"active"
            
            # link
            io.write(u"""<li class="%s"><a href="%s" class="%s">%s</a></li>""" % (highlight, 
                                                                                  self.PageUrl(page, usePageLink=not self.IsEditmode()), 
                                                                                  selected, 
                                                                                  page.GetTitle()))
            
            if highlight or selected:
                io = self._navigationLevel(page, level+1, active, path, io, ulclass)
        
        if len(pages) and level > 1:
            io.write(u"""</ul></li>\r\n""")
        return io


    # linked documentation, first start message 
    
    def doc(self, template=u"default.pt"):
        """
        renders the doc template
        """
        return render(u"nive.cms:doc/"+template, {u"context":self.context, u"view":self, u"request": self.request}, request=self.request)


    def IsEditmode(self):
        try:
            return self.request.editmode
        except:
            return False
        
    def HtmlTitle(self):
        t = self.request.environ.get(u"htmltitle")
        if not t:
            t = self.context.GetTitle()
        t2 = self.context.app.configuration.title
        return t2 + u" - " + t


    # bw 0.9.9
    @property
    def cmsview(self):
        return self.editorview

    
    





import copy

import test_MySql
try:
    from nive.utils.dataPool2.mySqlPool import *
except:
    pass



def sqlquery4(n):
    pool = MySql(test_MySql.conf)
    pool.SetStdMeta(copy.copy(test_MySql.stdMeta))
    pool.GetPoolStructureObj().SetStructure(test_MySql.struct)
    pool.CreateConnection(test_MySql.conn)

    print "SQL Query filename (text index) result=all, sort=filename, operator=like: ",
    t = time.time()
    for i in range(0,n):
        files = pool.SearchFiles({u"filename": u"file1.txt"}, sort=u"filename", operators={u"fielname":u"like"})
    t2 = time.time()

    pool.Close()
    print n, " queries in ", t2-t, "secs. ", (t2-t)/n, " per statement"
    print

def sqlquery5(n):
    pool = MySql(test_MySql.conf)
    pool.SetStdMeta(copy.copy(test_MySql.stdMeta))
    pool.GetPoolStructureObj().SetStructure(test_MySql.struct)
    pool.CreateConnection(test_MySql.conn)

    print "SQL Query filename (text index), result=all, sort=id, operator==: ",
    t = time.time()
    for i in range(0,n):
        files = pool.SearchFiles({u"filename": u"file1.txt"}, sort=u"id", operators={u"fielname":u"="})
    t2 = time.time()

    pool.Close()
    print n, " queries in ", t2-t, "secs. ", (t2-t)/n, " per statement"
    print

def sqlquery6(n):
    pool = MySql(test_MySql.conf)
    pool.SetStdMeta(copy.copy(test_MySql.stdMeta))
    pool.GetPoolStructureObj().SetStructure(test_MySql.struct)
    pool.CreateConnection(test_MySql.conn)

    print "SQL Query filename (text index) no result: ",
    t = time.time()
    for i in range(0,n):
        files = pool.SearchFiles({u"filename": u"filexxx.txt"}, sort=u"filename", operators={u"fielname":u"like"})
    t2 = time.time()

    pool.Close()
    print n, " queries in ", t2-t, "secs. ", (t2-t)/n, " per statement"
    print






def run():
    n = 100
    sqlquery4(n)
    sqlquery5(n)
    sqlquery6(n)




if __name__ == '__main__':
    run()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
This file provides global *search* functionality, database lookup and sql query 
wrappers. It is usually attached to root objects.

Search parameter handling
--------------------------
Most functions support a similar parameter handling for generating sql queries.  

===========  ===========================================================================
Parameters
===========  ===========================================================================
fields       list of database fields (field ids) to include in result.
             prefix '-': special fields or aggregate functions can be inserted by adding 
                         a '-' in front of the field. -count(*) or -MAX(field1)
             prefix '+': special fields skipped in SQL query e.g. '+preview'
parameter    dictionary with fieldname:value entries used for search conditions
operators    dictionary with fieldname:operator entries used for search conditions
             default: strings=*LIKE*, all others='='
             possible values: ``=, LIKE, IN, >, <, <=, >=, !=, BETWEEN ``
sort         result sort field or list if multiple
ascending    sort ascending or decending
start        start position in result
max          maximum number of result records
===========  ===========================================================================

Parameter for operator BETWEEN has to be formatted: e.g. '2006/10/10' AND '2006/10/11'

================  =====================================================================
Keyword options   (Not supported by all functions)
================  =====================================================================
logicalOperator   link between conditions. default: *AND*. possible: ``AND, OR, NOT``
condition         custom sql condition statement appended to WHERE clause 
groupby           used as sql GROUP BY statement
join              adds a custom custom sql join statement
jointype          left, inner, right. default = inner (used by ``SearchType()``)
mapJoinFld        fieldname to map title by left/right join (used by ``SearchType()``)
addID             Add fld.id in query without showing in result list. used for custom 
                  columns e.g. -(select ...)
skipRender        render result flds as html element. 
                  default: ``pool_type, pool_wfa, pool_wfp`` 
                  to skip all: *True*, or a list of fields  ("pool_wfa","pool_type")
skipCount         enable or disable second query to get the number of all records. 
================  =====================================================================


=============  ========================================================================
Search result  (supported by Search*() functions)
=============  ========================================================================
criteria       dictionary containing parameters used in search
count          number of records contained in result
total          total number of records matching the query
items          result list. each record as dictionary. Each field is rendered for 
               display depending on *skipRender* keyword. This means list entries are
               replaced by their readable names, dates are rendered readable. 
start          record start number in result 
max            maximum number of records 
next           start number of next record set
nextend        end number of next record set
prev           start number of previous record set
prevend        end number of previous record set
sql            the sql statement used
=============  ========================================================================

"""

import time

from nive.utils.utils import ConvertToNumberList
from nive.utils.language import LanguageExtension, CountryExtension
from nive.views import FieldRenderer
from nive.security import GetUsers
from nive.definitions import IFieldConf
from nive.definitions import FieldConf
from nive.definitions import ConfigurationError

class Search:
    """    """


    # Simple search functions ----------------------------------------------------------------------------------------------
    
    def Select(self, pool_type=None, parameter={}, fields=["id"], operators={}, sort=None, ascending = 1, start=0, max=0, **kw):
        """
        Fast and simple sql query. 
        
        If *pool_type* is not set the query will apply to the meta layer only. In this case you can
        only include pool_meta.fields in *fields*. To use a single table pass *dataTable* as keyword.
        
        If *pool_type* is set you can use meta and data *fields*. The query is restricted to a single 
        type.
        
        Supported keywords: ``groupby, logicalOperator, condition, dontAddType, dataTable``
        
        The following example selects all children of the current object ::
            
            fields = ["id", "title", "pool_type"]
            parameter["pool_unitref"] = self.id
            records = self.root().Select(parameter=parameter, fields=fields)
        
        returns records as list
        """
        if not fields:
            raise TypeError, "No fields specified"
        db = self.db
        if pool_type==None:
            dataTable=kw.get("dataTable") or u"pool_meta"
            sql, values = db.FmtSQLSelect(fields, parameter, dataTable=dataTable, singleTable=1, operators=operators, sort=sort, ascending=ascending, start=start, max=max, groupby=kw.get("groupby"), logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"))
        else:
            if not parameter.has_key("pool_type") and not kw.get("dontAddType"):
                parameter["pool_type"] = pool_type
            if not operators.has_key("pool_type"):
                operators["pool_type"] = u"="
            typeInf = self.app.GetObjectConf(pool_type)
            if not typeInf:
                raise ConfigurationError, pool_type + " type not found"
            sql, values = db.FmtSQLSelect(fields, parameter, dataTable=typeInf["dbparam"], operators=operators, sort=sort, ascending=ascending, start=start, max=max, groupby=kw.get("groupby"), logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"))
        recs = db.Query(sql, values)
        return recs


    def SelectDict(self, pool_type=None, parameter={}, fields=[u"id"], operators={}, sort=None, ascending = 1, start=0, max=0, **kw):
        """
        Fast and simple sql query. 
        
        If *pool_type* is not set the query will apply to the meta layer only. In this case you can
        only include pool_meta.fields in *fields*. 
        
        If *pool_type* is set you can use meta and data *fields*. The query is restricted to a single 
        type.
        
        Supported keywords: ``groupby, logicalOperator, condition``
        
        Records are returned as dictionaries
        
        The following example selects all children *not* of type *image* of the current object ::
            
            fields = ["id", "title", "pool_type"]
            parameter = {"pool_unitref": self.id}
            operators = {"pool_type": "!="}
            records = self.root().SelectDict("image", parameter=parameter, fields=fields, 
                                             operators=operators)
        
        returns records as dict list
        """
        recs = self.Select(pool_type, parameter, fields, operators, sort, ascending, start, max, **kw)
        if len(recs)==0:
            return recs
        # convert
        if len(fields) > len(recs[0]):
            raise TypeError, "Too many fields"
        for i in range(len(fields)):
            name = fields[i]
            if name.find(u" as ")!=-1:
                name = name.split(u" as ")[1]
                fields[i] = name
        return [dict(zip(fields, r)) for r in recs]


    # Extended search functions ----------------------------------------------------------------------------------------------

    def Search(self, parameter, fields = [], operators = {}, sort = u"title", ascending = 1, start = 0, max = 100, **kw):
        """
        Extended meta layer search function. Supports all keyword options and search result. 
        
        Example ::
        
            root.()Search({"title":"test"}, 
                          fields=["id", "pool_type", "title"], 
                          start=0, max=50, 
                          operators={"title":"="})
        
        returns search result (See above)
        """
        t = time.time()

        # check parameter
        try:    ascending = int(ascending)
        except:    ascending = 1
        try:    start = int(start)
        except:    start = 0
        try:    max = int(max)
        except:    max = 100
        debug = kw.get("debug",False)

        # return empty result set and not all reords
        if kw.get("showAll",True) == False and parameter == {}:
            parameter["id"] = 0

        # lookup field definitions
        fields = self._GetFieldDefinitions(fields)
        fldList = []
        groupcol = 0
        for f in fields:
            if f["id"][0] == "-":
                groupcol += 1
            fldList.append(f["id"])

        # add id. required for group by  queries
        removeID = False
        if (not "id" in fldList and groupcol == 0 and kw.get("groupby") == None) or kw.get("addID")==1:
            fldList.append("id")
            fields.append(self.app.GetFld("id"))
            removeID = True

        db = self.db
        if not db:
            raise ConnectionError, "No database connection"

        sql, values = db.FmtSQLSelect(fldList, parameter=parameter, operators=operators, 
                              sort=sort, ascending=ascending, start=start, max=max, 
                              groupby=kw.get("groupby"), 
                              logicalOperator=kw.get("logicalOperator"), 
                              condition=kw.get("condition"), 
                              join=kw.get("join"))
        records = db.Query(sql, values)

        # prepare field renderer
        skipRender = kw.get("skipRender", False)
        renderer = None
        if not skipRender:
            # default values
            renderer = FieldRenderer(self, skip = ["pool_type", "pool_wfa", "pool_wfp"])
        elif isinstance(skipRender, (list,tuple)):
            renderer = FieldRenderer(self, skip = skipRender)

        # parse alias field names used in sql query
        p = 0
        for f in fldList:
            if f[0] == u"-" and f.find(u" as ") != -1:
                a = f.split(u" as ")[-1]
                a = a.replace(u" ", u"")
                a = a.replace(u")", u"")
                fldList[p] = a
            p += 1

        # convert records
        items = []
        cnt = 0
        total = 0
        for rec in records:
            cnt+=1
            # render if activated
            if renderer:
                rec2 = []
                for p in range(len(fields)):
                    rec2.append(renderer.Render(fields[p], rec[p], False, **kw))
                rec = rec2
            items.append(dict(zip(fldList, rec)))
            if max > 0 and cnt == max:
                break

        # total records
        total = len(items)
        if total == max and kw.get("skipCount") != 1:
            if not kw.get("groupby"):
                sql2, values =db.FmtSQLSelect([u"-count(*)"], parameter=parameter, operators=operators, 
                                      sort=sort, ascending=ascending, start=None, max=None, 
                                      logicalOperator=kw.get("logicalOperator"), 
                                      condition=kw.get("condition"), 
                                      join=kw.get("join"))
                total = db.Query(sql2, values)[0][0]
            else:
                sql2, values = db.FmtSQLSelect([u"-count(DISTINCT %s)" % (kw.get("groupby"))], parameter=parameter, operators=operators, 
                                       sort=sort, ascending=ascending, start=None, max=None, 
                                       logicalOperator=kw.get("logicalOperator"), 
                                       condition=kw.get("condition"), 
                                       join=kw.get("join"))
                total = db.Query(sql2, values)[0][0]

        # prepare result dictionary and paging information
        result = {}
        result["criteria"] = parameter
        result["count"] = cnt
        result["total"] = total
        result["items"] = items
        result["time"] = time.time() - t
        result["start"] = start
        result["max"] = max
        if debug:
            result["sql"] = sql
        
        next = start + max
        if next >= total:
            next = 0
        result["next"] = next
        nextend = next + max
        if nextend > total:
            nextend = total
        result["nextend"] = nextend

        prev = start - max
        if prev < 1:
            prev = 0
        result["prev"] = prev
        prevend = prev + max
        result["prevend"] = prevend

        return result


    def SearchType(self, pool_type, parameter, fields = [], sort = u"title", ascending = 1, start = 0, max = 100, **kw):
        """
        Extended meta and data layer search function. Supports all keyword options and search result. 
        
        Example ::
        
            root.()SearchType("image", 
                              parameter={"title":"test"}, 
                              fields=["id", "pool_type", "title"], 
                              start=0, max=50, 
                              operators={"title":"="})
        
        returns search result (See above)
        """
        t = time.time()

        # check parameter
        try:    ascending = int(ascending)
        except:    ascending = 1
        try:    start = int(start)
        except:    start = 0
        try:    max = int(max)
        except:    max = 100

        # set join type
        default_join = 0
        if not kw.has_key("jointype") or kw.get("jointype")==u"inner":
            default_join = 1
            parameter["pool_type"] = pool_type

        if kw.get("showAll") == False and parameter == {}:
            parameter["id"] = 0

        # update result field list, search for special fields
        fields = self._GetFieldDefinitions(fields, pool_type)
        fldList = []
        groupcol = 0
        for f in fields:
            if f["id"][0] == u"-":
                groupcol += 1
            fldList.append(f["id"])

        # add id fld
        removeID = False
        # groupcol == 0 entfernt
        #if not "id" in fldList and kw.get("groupby") == None:
        if (not "id" in fldList and groupcol == 0 and kw.get("groupby") == None) or kw.get("addID")==1:
            fldList.append("id")
            fields.append(self.app.GetFld("id"))
            removeID = True

        #operators
        operators=kw.get("operators")
        if not operators:
            operators = {}
        if not operators.has_key("pool_type"):
            operators["pool_type"] = u"="
        if not default_join:
            operators["jointype"] = kw.get("jointype")

        items = []
        cnt = 0
        total = 0
        sql = ""

        typeInf = self.app.GetObjectConf(pool_type)
        if not typeInf:
            raise ConfigurationError, "Type not found (%s)" % (pool_type)

        db = self.db
        if not db:
            ct = 0
        else:

            sql, values = db.FmtSQLSelect(fldList, parameter=parameter, sort=sort, ascending=ascending, dataTable=typeInf["dbparam"], start=start, max=max, operators=operators, groupby=kw.get("groupby"), logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"), join=kw.get("join"), mapJoinFld=kw.get("mapJoinFld"))
            aL = db.Query(sql, values)
            
            # render
            converter = FieldRenderer(self)
            skipRender = kw.get("skipRender", False)
            if skipRender == True:
                skipRender = fldList
            elif not skipRender:
                skipRender = [u"pool_type", u"pool_wfa", u"pool_wfp"]

            # convert records
            p = 0
            for f in fldList:
                if f[0] == u"-" and f.find(u" as ") != -1:
                    a = f.split(u" as ")[-1]
                    a = a.replace(u" ", u"")
                    a = a.replace(u")", u"")
                    fldList[p] = a
                p += 1
            for aI in aL:
                cnt+=1
                # convert
                aI2 = []
                for p in range(len(fields)):
                    aI2.append(converter.Render(fields[p], aI[p], False, render = (fldList[p] not in skipRender), **kw))
                items.append(dict(zip(fldList, aI2)))

                if max > 0 and cnt == max:
                    break

            # total records
            if len(items) == max and kw.get("skipCount") != 1:
                if not kw.get("groupby"):
                    sql2, values = db.FmtSQLSelect([u"-count(*) as cnt"], parameter=parameter, sort=u"-cnt", ascending=ascending, dataTable=typeInf["dbparam"], start=None, max=None, operators=operators, logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"), join=kw.get("join"))
                    total = db.Query(sql2, values)[0][0]
                else:
                    sql2, values = db.FmtSQLSelect([u"-count(DISTINCT %s) as cnt" % (kw.get("groupby"))], parameter=parameter, sort="-cnt", ascending=ascending, dataTable=typeInf["dbparam"], start=None, max=None, operators=operators, logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"), join=kw.get("join"))
                    total = db.Query(sql2, values)[0][0]
            else:
                total = len(items) + start

        result = {}
        result["criteria"] = parameter
        result["count"] = cnt
        result["total"] = total
        result["items"] = items
        result["time"] = time.time() - t
        result["start"] = start
        result["max"] = max
        result["sql"] = sql
        
        next = start + max
        if next >= total:
            next = 0
        result["next"] = next
        nextend = next + max
        if nextend > total:
            nextend = total
        result["nextend"] = nextend

        prev = start - max
        if prev < 1:
            prev = 0
        result["prev"] = prev
        prevend = prev + max
        result["prevend"] = prevend

        return result


    def SearchData(self, pool_type, parameter, fields = [], sort = u"id", ascending = 1, start = 0, max = 100, **kw):
        """
        Extended data layer search function. Supports all keyword options and search result. 
        
        Example ::
        
            root.()SearchData("image", 
                              parameter={"text": "new"}, 
                              fields=["id", "text", "title"], 
                              start=0, max=50, 
                              operators={"text":"LIKE"})
        
        returns search result (See above)
        """
        t = time.time()

        # check parameter
        try:    ascending = int(ascending)
        except:    ascending = 1
        try:    start = int(start)
        except:    start = 0
        try:    max = int(max)
        except:    max = 100

        if kw.get("showAll",True) == False and parameter == {}:
            parameter["id"] = 0

        # update result field list, search for special fields
        fields = self._GetFieldDefinitions(fields, pool_type)
        fldList = []
        groupcol = 0
        for f in fields:
            if f["id"][0] == u"-":
                groupcol += 1
            fldList.append(f["id"])

        # add id fld
        removeID = False
        # groupcol == 0 entfernt
        #if not "id" in fldList and kw.get("groupby") == None:
        if (not "id" in fldList and groupcol == 0 and kw.get("groupby") == None) or kw.get("addID")==1:
            fldList.append("id")
            fields.append(self.app.GetFld("id", pool_type))
            removeID = True

        #operators
        operators=kw.get("operators")
        if not operators:
            operators = {}

        items = []
        cnt = 0
        total = 0
        sql = ""

        typeInf = self.app.GetObjectConf(pool_type)
        if not typeInf:
            raise ConfigurationError, "Type not found (%s)" % (pool_type)

        db = self.db
        if not db:
            ct = 0
        else:
            sql, values = db.FmtSQLSelect(fldList, parameter=parameter, dataTable=typeInf["dbparam"], sort=sort, ascending=ascending, start=start, max=max, operators=operators, groupby=kw.get("groupby"), logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"), singleTable=1)
            aL = db.Query(sql, values)
            
            # render
            converter = FieldRenderer(self)
            skipRender = kw.get("skipRender", False)
            if skipRender == True:
                skipRender = fldList
            else:
                skipRender = []

            # convert records
            p = 0
            for f in fldList:
                if f[0] == u"-" and f.find(u" as ") != -1:
                    a = f.split(u" as ")[-1]
                    a = a.replace(u" ", u"")
                    a = a.replace(u")", u"")
                    fldList[p] = a
                p += 1
            for aI in aL:
                cnt+=1
                # convert
                aI2 = []
                for p in range(len(fields)):
                    aI2.append(converter.Render(fields[p], aI[p], False, render = (fldList[p] not in skipRender), **kw))
                items.append(dict(zip(fldList, aI2)))

                if max > 0 and cnt == max:
                    break

            # total records
            if len(items) == max and kw.get("skipCount") != 1:
                if not kw.get("groupby"):
                    sql2, values = db.FmtSQLSelect([u"-count(*) as cnt"], parameter=parameter, dataTable=typeInf["dbparam"], sort=u"-cnt", ascending=ascending, start=None, max=None, operators=operators, logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"), singleTable=1)
                    total = db.Query(sql2, values)[0][0]
                else:
                    sql2, values = db.FmtSQLSelect([u"-count(DISTINCT %s) as cnt" % (kw.get("groupby"))], parameter=parameter, dataTable=typeInf["dbparam"], sort="-cnt", ascending=ascending, start=None, max=None, operators=operators, logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"), singleTable=1)
                    total = db.Query(sql2, values)[0][0]
            else:
                total = len(items) + start

        result = {}
        result["criteria"] = parameter
        result["count"] = cnt
        result["total"] = total
        result["items"] = items
        result["time"] = time.time() - t
        result["start"] = start
        result["max"] = max
        result["sql"] = sql

        next = start + max
        if next >= total:
            next = 0
        result["next"] = next
        nextend = next + max
        if nextend > total:
            nextend = total
        result["nextend"] = nextend

        prev = start - max
        if prev < 1:
            prev = 0
        result["prev"] = prev
        prevend = prev + max
        result["prevend"] = prevend

        return result


    def SearchFulltext(self, phrase, parameter=None, fields = ("id","title","pool_type","-pool_fulltext.text as fulltext"), sort = u"", ascending = 1, start = 0, max = 300, **kw):
        """
        Fulltext search function. Searches all text fields marked for fulltext search. Uses *searchPhrase* 
        as parameter for text search. Supports all keyword options and search result. 
        
        Example ::
        
            root().SearchFulltext("new", parameter={}, 
                              fields=["id", "title"], 
                              start=0, max=50, 
                              operators={"text":"LIKE"})
        
        returns search result (See above)
        """
        t = time.time()

        # check parameter
        if not parameter:
            parameter = {}
        if phrase==None:
            phrase = u""
        searchFor = phrase
        try:    ascending = int(ascending)
        except:    ascending = 1
        try:    start = int(start)
        except:    start = 0
        try:    max = int(max)
        except:    max = 100

        fields = self._GetFieldDefinitions(fields)

        fldList = []

        if kw.get("showAll",True) == False and parameter == {} and phrase == "":
            parameter["id"] = 0

        groupcol = 0
        for f in fields:
            if f["id"][0] == "-":
                groupcol += 1
            fldList.append(f["id"])

        # add id
        removeID = False
        if not "id" in fldList and groupcol == 0:
            fldList.append("id")
            fields.append(self.app.GetFld("id"))
            removeID = True

        if phrase.find(u"*") == -1:
            phrase = u"%%%s%%" % phrase
        else:
            phrase = phrase.replace(u"*", u"%")

        items = []
        cnt = 0
        total = 0
        db = self.db
        if not db:
            ct = 0
        else:
            sql, values = db.GetFulltextSQL(phrase, fldList, parameter, sort=sort, ascending=ascending, start=start, max=max, operators=kw.get("operators",{}), logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"), join=kw.get("join"))
            aL = db.Query(sql, values)
            
            # render
            converter = FieldRenderer(self)
            skipRender = kw.get("skipRender", False)
            if skipRender == True:
                skipRender = fldList
            elif not skipRender:
                skipRender = ["pool_type", "pool_wfa", "pool_wfp"]
            p = 0
            for f in fldList:
                if f[0] == "-" and f.find(u" as ") != -1:
                    a = f.split(u" as ")[-1]
                    a = a.replace(u" ", u"")
                    a = a.replace(u")", u"")
                    fldList[p] = a
                p += 1
            # convert records
            for aI in aL:
                cnt+=1
                aI2 = []
                for p in range(len(fields)):
                    aI2.append(converter.Render(fields[p], aI[p], False, render = (fldList[p] not in skipRender), **kw))
                items.append(dict(zip(fldList, aI2)))
                if max > 0 and cnt == max:
                    break

            # total records
            sql2, values = db.GetFulltextSQL(phrase, [u"-count(*)"], parameter, sort=sort, ascending=ascending, start=None, max=None, operators=kw.get("operators",{}), skipRang=1, logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"), join=kw.get("join"))
            total = db.Query(sql2, values)[0][0]
            
        result = {}
        result["phrase"] = searchFor
        result["criteria"] = parameter
        result["count"] = cnt
        result["total"] = total
        result["items"] = items
        result["time"] = time.time() - t
        result["start"] = start
        result["max"] = max
        result["sql"] = sql

        next = start + max
        if next >= total:
            next = 0
        result["next"] = next
        nextend = next + max
        if nextend > total:
            nextend = total
        result["nextend"] = nextend

        prev = start - max
        if prev < 1:
            prev = 0
        result["prev"] = prev
        prevend = prev + max
        result["prevend"] = prevend

        return result


    def SearchFulltextType(self, pool_type, phrase, parameter=None, fields = ["id","title"], sort = None, ascending = 1, start = 0, max = 300, **kw):
        """
        Fulltext search function. Searches all text fields marked for fulltext search of the given type. Uses *searchPhrase* 
        as parameter for text search. Supports all keyword options and search result. 
        
        Example ::
        
            root.()SearchFulltextType("text",
                              parameter={"searchPhrase": "new"}, 
                              fields=["id", "text", "title"], 
                              start=0, max=50, 
                              operators={"text":"LIKE"})
        
        returns search result (See above)
        """
        t = time.time()

        # check parameter
        if not parameter:
            parameter = {}
        if phrase==None:
            phrase = u""
        searchFor = phrase
        try:    ascending = int(ascending)
        except:    ascending = 1
        try:    start = int(start)
        except:    start = 0
        try:    max = int(max)
        except:    max = 100

        # set join type
        default_join = 0
        if not kw.has_key("jointype") or kw.get("jointype")==u"inner":
            default_join = 1
            parameter["pool_type"] = pool_type

        if kw.get("showAll",True) == False and parameter == {} and phrase == "":
            parameter["id"] = 0

        # update result field list, search for special fields
        fields = self._GetFieldDefinitions(fields, pool_type)
        fldList = []
        groupcol = 0
        for f in fields:
            if f["id"][0] == u"-":
                groupcol += 1
            fldList.append(f["id"])

        # add id
        removeID = False
        if (not "id" in fldList and groupcol == 0 and kw.get("groupby") == None) or kw.get("addID")==1:
            fldList.append(u"id")
            fields.append(self.app.GetFld("id", pool_type))
            removeID = True

        #operators
        operators=kw.get("operators")
        if not operators:
            operators = {}
        if not operators.has_key("pool_type"):
            operators["pool_type"] = u"="
        if not default_join:
            operators["jointype"] = kw.get("jointype")

        # fulltext wildcard
        if phrase.find(u"*") == -1:
            phrase = u"%%%s%%" % phrase
        else:
            phrase = phrase.replace(u"*", u"%")

        typeInf = self.app.GetObjectConf(pool_type)
        if not typeInf:
            raise ConfigurationError, "Type not found (%s)" % (pool_type)

        items = []
        cnt = 0
        total = 0
        sql = ""

        db = self.db
        if not db:
            ct = 0
        else:
            sql, values = db.GetFulltextSQL(phrase, fldList, parameter, dataTable=typeInf["dbparam"], sort=sort, ascending=ascending, start=start, max=max, operators=operators, groupby=kw.get("groupby"), logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"), join=kw.get("join"), mapJoinFld=kw.get("mapJoinFld"))
            aL = db.Query(sql, values)
            
            # render
            converter = FieldRenderer(self)
            skipRender = kw.get("skipRender", False)
            if skipRender == True:
                skipRender = fldList
            elif not skipRender:
                skipRender = ["pool_type", "pool_wfa", "pool_wfp"]

            # convert records
            p = 0
            for f in fldList:
                if f[0] == u"-" and f.find(u" as ") != -1:
                    a = f.split(u" as ")[-1]
                    a = a.replace(u" ", u"")
                    a = a.replace(u")", u"")
                    fldList[p] = a
                p += 1
            for aI in aL:
                cnt+=1
                aI2 = []
                for p in range(len(fields)):
                    aI2.append(converter.Render(fields[p], aI[p], False, render = (fldList[p] not in skipRender), **kw))
                items.append(dict(zip(fldList, aI2)))
                if max > 0 and cnt == max:
                    break

            # total records
            if len(items) == max and kw.get("skipCount") != 1:
                if not kw.get("groupby"):
                    sql2, values = db.GetFulltextSQL(phrase, [u"-count(*) as cnt"], parameter, dataTable=typeInf["dbparam"], ascending=ascending, start=None, max=None, operators=operators, skipRang=1, logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"), join=kw.get("join"))
                    total = db.Query(sql2, values)[0][0]
                else:
                    sql2, values = db.GetFulltextSQL(phrase, [u"-count(DISTINCT %s) as cnt" % (kw.get("groupby"))], parameter, dataTable=typeInf["dbparam"], ascending=ascending, start=None, max=None, operators=operators, skipRang=1, logicalOperator=kw.get("logicalOperator"), condition=kw.get("condition"), join=kw.get("join"))
                    total = db.Query(sql2, values)[0][0]
            else:
                total = len(items) + start
            
        result = {}
        result["phrase"] = searchFor
        result["criteria"] = parameter
        result["count"] = cnt
        result["total"] = total
        result["items"] = items
        result["time"] = time.time() - t
        result["start"] = start
        result["max"] = max
        result["sql"] = sql
        
        next = start + max
        if next >= total:
            next = 0
        result["next"] = next
        nextend = next + max
        if nextend > total:
            nextend = total
        result["nextend"] = nextend

        prev = start - max
        if prev < 1:
            prev = 0
        result["prev"] = prev
        prevend = prev + max
        result["prevend"] = prevend

        return result


    def SearchFilename(self, filename, parameter, fields = [], sort = None, ascending = 1, start = 0, max = 100, **kw):
        """
        Filename search function. Searches all physical file filenames (not url path names). Supports all 
        keyword options and search result. 
        
        Includes matchinng files as "result_files" in each record.
        
        Example ::
        
            root.()SearchFulltextType("text",
                              parameter={"searchPhrase": "new"}, 
                              fields=["id", "text", "title"], 
                              start=0, max=50, 
                              operators={"text":"LIKE"})
        
        returns search result (See above)
        """
        db = self.db
        if kw.get("showAll",True) == False and filename == "":
            files = []
        else:
            files = db.SearchFilename(filename)

        ids = []
        filesMapped = {}
        for f in files:
            if not f["id"] in ids:
                ids.append(f["id"])
            if not filesMapped.has_key(str(f["id"])):
                filesMapped[str(f["id"])] = []
            filesMapped[str(f["id"])].append(f)
        if len(ids)==0:
            ids.append(0)
        
        if not kw.has_key("operators"):
            kw["operators"] = {}
        parameter["id"] = ids
        kw["operators"].update({"id":u"IN"})
        result = self.Search(parameter=parameter, fields=fields, sort=sort, ascending=ascending, start=start, max=max, **kw)

        # merge result and files
        for rec in result["items"]:
            if not filesMapped.has_key(str(rec["id"])):
                continue
            rec["result_files"] = filesMapped[rec["id"]]
            # copy first file in list
            file = filesMapped[str(rec["id"])][0]
            rec["+size"] = file["size"]
            rec["+extension"] = file["extension"]
            rec["+filename"] = file["filename"]
            rec["+fileid"] = file["fileid"]
            rec["+key"] = file["filekey"]

        # update result
        del result["criteria"]["id"]
        result["criteria"]["filename"] = filename
        return result



    # Tree structure -----------------------------------------------------------

    def GetTree(self, flds=[u"id", u"pool_unitref", u"title", u"pool_filename", u"pool_type", u"pool_state", u"pool_wfa", u"pool_sort"], sort=u"pool_sort", base=0, parameter=u""):
        """
        Select list of all folders from db.
        
        returns the subtree
        {'items': [{u'id': 354956L, 'ref1': 354954L, 'ref2': 354952L, ..., 'ref10': None, 'items': [...]}]
        """
        db = self.db
        return db.GetTree(flds=flds, sort=sort, base=base, parameter=parameter)


    def TreeParentIDs(self, id):
        """
        returns the parent ids for the object with id as list
        """
        db = self.db
        return db.GetParentPath(id)


    def TreeParentTitles(self, id):
        """
        returns the parent titles for the object with id as list
        """
        db = self.db
        return db.GetParentTitles(id)


    # Codelists representation for entries -----------------------------------------------------------------------------------------

    def GetEntriesAsCodeList(self, pool_type, name_field, parameter=None, operators=None, sort = None):
        """
        Search the database for entries of type *pool_type* and return matches as codelist ::
        
            [{"name": name_field, "id": object.id}, ... ]
        
        If the name_field is stored as data field, insert *data.* at the beginning of name_field 
        (e.g. ``data.header``)
        
        returns list
        """
        if parameter==None:
            parameter = {}
        if operators==None:
            operators = {}
        if not sort:
            sort = name_field
        if not parameter.has_key(u"pool_type"):
            parameter[u"pool_type"] = pool_type
        recs = self.SelectDict(pool_type=pool_type, parameter=parameter, fields=[u"id", u"pool_unitref", name_field+u" as name"], operators=operators, sort = sort)
        return recs


    def GetEntriesAsCodeList2(self, name_field, parameter=None, operators=None, sort = None):
        """
        Search the database and return matches as codelist ::
        
            [{"name": name_field, "id": object.id}, ... ]
        
        returns list
        """
        if parameter==None:
            parameter = {}
        if operators==None:
            operators = {}
        if not sort:
            sort = name_field
        recs = self.SelectDict(parameter=parameter, fields=[u"id", u"pool_unitref", name_field+u" as name"], operators=operators, sort = sort)
        return recs


    def GetGroupAsCodeList(self, pool_type, name_field, parameter=None, operators=None, sort = None):
        """
        Search the database for entries of type *pool_type* and return matches grouped by unique
        *name_field* values as codelist ::
        
            [{"name": name_field, "id": object.id}, ... ]
        
        If the name_field is stored as data field, insert *data.* at the beginning of name_field 
        (e.g. ``data.header``)
        
        returns list
        """
        if parameter==None:
            parameter = {}
        if operators==None:
            operators = {}
        if not sort:
            sort = name_field
        if not parameter.has_key(u"pool_type"):
            parameter[u"pool_type"] = pool_type
        recs = self.SelectDict(pool_type=pool_type, parameter=parameter, fields=[u"id", u"pool_unitref", name_field+u" as name"], operators=operators, groupby = name_field, sort = sort)
        return recs


    def GetGroupAsCodeList2(self, name_field, parameter=None, operators=None, sort = None):
        """
        Search the database and return matches grouped by unique *name_field* values as codelist ::
        
            [{"name": name_field, "id": object.id}, ... ]
        
        returns list
        """
        if parameter==None:
            parameter = {}
        if operators==None:
            operators = {}
        if not sort:
            sort = name_field
        recs = self.SelectDict(parameter=parameter, fields=[u"id", u"pool_unitref", name_field+u" as name"], operators=operators, groupby = name_field, sort = sort)
        return recs


    # Name / ID lookup -----------------------------------------------------------

    def FilenameToID(self, filename, unitref=None, parameter=None, firstResultOnly=True, operators=None):
        """
        Convert url path filename (meta.pool_filename) to id. This function does not lookup
        physical files and their filenames.

        returns id
        """
        if parameter==None:
            parameter = {}
        if operators==None:
            operators = {}
        if unitref != None:
            parameter[u"pool_unitref"] = unitref
        parameter[u"pool_filename"] = filename
        operators[u"pool_filename"] = u"="
        recs = self.Select(parameter=parameter, fields=[u"id",u"pool_type",u"pool_unitref",u"title"], operators=operators)
        #print recs
        if firstResultOnly:
            if len(recs) == 0:
                return 0
            return recs[0][0]
        return l


    def IDToFilename(self, id):
        """
        Convert id to url path filename (meta.pool_filename). This function does not lookup
        physical files and their filenames.
        
        returns string
        """
        parameter={"id": id}
        recs = self.Select(parameter=parameter, fields=[u"pool_filename"])
        if len(recs) == 0:
            return u""
        return recs[0][0]


    def ConvertDatarefToID(self, pool_type, dataref):
        """
        Search for object id based on dataref and pool_type.

        returns id
        """
        parameter={u"pool_type": pool_type, u"pool_dataref": dataref}
        recs = self.Select(parameter=parameter, fields=[u"id"])
        if len(recs) == 0:
            return 0
        return recs[0][0]


    def GetMaxID(self):
        """
        Lookup id of the last created object.

        returns id
        """
        recs = self.Select(fields=[u"-max(id)"])
        if len(recs) == 0:
            return 0
        return recs[0][0]


    # References --------------------------------------------------------------------

    def GetReferences(self, unitID, types = []):
        """
        Search for references in unit or unitlist fields of all objects.
        
        returns id list
        """
        if types == []:
            types = self.app.GetAllObjectConfs()
        else:
            l = []
            for t in types:
                l.append(self.app.GetObjectConf(t))
            types = l
        references = []
        ids = [unitID]

        db = self.db

        # search unit flds
        for t in types:
            for f in t["data"]:
                if f["datatype"] != "unit":
                    continue
                l = self.Select(pool_type=t["id"], parameter={f["id"]:unitID}, fields=["id", "title", "pool_type"], sort="pool_sort")
                for r in l:
                    if not r[0] in ids:
                        ids.append(r[0])
                        references.append(r)

        # search unitlist flds
        for t in types:
            for f in t["data"]:
                if f["datatype"] != "unitlist":
                    continue
                l = self.Select(pool_type=t["id"], parameter={f["id"]:str(unitID)}, fields=["id", "title", "pool_type", f["id"]], sort="pool_sort", operators={f["id"]:"LIKE"})
                for r in l:
                    if not r[2] == t["id"]:
                        continue
                    unitRefs = ConvertToNumberList(r[3])
                    if not unitID in unitRefs:
                        continue
                    if not r[0] in ids:
                        ids.append(r[0])
                        references.append(r)

        return references



    # Field list items ------------------------------------------

    def LoadListItems(self, fieldconf, obj=None, pool_type=None, force=False):
        """
        Load field list items in correspondance to to field.id and field.settings.
        if force is false and fieldconf contains list items, the existing 
        field.listItems are returned. set force=true to reload.
        
        obj and pool_type only used for workflow lookup
        
        returns dict list
        """
        values = []
        if not fieldconf:
            return values

        if fieldconf.listItems and not force:
            # skip loading if list filled
            if hasattr(fieldconf.listItems, '__call__'):
                return fieldconf.listItems(fieldconf, obj or self)
            return fieldconf.listItems

        fld = fieldconf.id
        # load list items

        if fieldconf.settings:
            # settings dyn list
            dyn = fieldconf.settings.get("codelist")
            if not dyn:
                pass
            elif dyn == "users":
                return GetUsers(self.app)
            elif dyn == "groups":
                portal = self.app.portal
                if portal==None:
                    portal = self.app
                return portal.GetGroups(sort="name", visibleOnly=True)
            elif dyn == "localgroups":
                return self.app.GetGroups(sort="name", visibleOnly=True)
            elif dyn == "languages":
                return LanguageExtension().Codelist()
            elif dyn == "countries":
                return CountryExtension().Codelist()
            elif dyn == "types":
                return self.app.GetAllObjectConfs()
            elif dyn == "categories":
                return self.app.GetAllCategories()
            elif dyn[:5] == "type:":
                type = dyn[5:]
                return self.GetEntriesAsCodeList(type, "title", parameter= {}, operators = {}, sort = "title")
            elif dyn == "meta":
                return self.GetEntriesAsCodeList2("title", parameter= {}, operators = {}, sort = "title")

        if fld == "pool_type":
            values = self.app.GetAllObjectConfs()

        elif fld == "pool_category":
            values = self.app.GetAllCategories()

        elif fld == "pool_groups":
            local = fieldconf.settings.get("local")
            loader = self.app
            if not local:
                portal = self.app.portal
                if portal:
                    loader = portal
            values = loader.GetGroups(sort="name", visibleOnly=True)

        elif fld == "pool_language":
            values = self.app.GetLanguages()

        elif fld == "pool_wfa":
            # uses type object as param
            if obj:
                try:
                    aWfp = obj.meta.get("pool_wfp")
                    obj = self.app.GetWorkflow(aWfp)
                    if obj:
                        values = obj.GetActivities()
                except:
                    pass
            elif pool_type:
                aWfp = self.app.GetObjectConf(pool_type).get("workflowID")
                try:
                    obj = self.app.GetWorkflow(aWfp)
                    values = obj.GetActivities()
                except:
                    pass
            else:
                values = []

        elif fld == "pool_wfp":
            values = self.app.GetAllWorkflowConfs()

        return values





    # Internal functions--------------------------------------------------------------------

    def _GetFieldDefinitions(self, fields, pool_type = None):
        f = []
        for fld in fields:
            if IFieldConf.providedBy(fld):
                f.append(fld)
                continue
            if not isinstance(fld, basestring):
                continue
            if fld in ("__preview__",):
                continue
            # skip custom flds
            if fld[0] == "+":
                continue
            # Aggregate functions, custom flds
            if fld[0] == "-":
                f.append(FieldConf(**{"id": fld, "name": fld, "datatype": "string"}))
                continue

            fl = self.app.GetFld(fld, pool_type)
            if fl:
                f.append(fl)
        return f



# -*- coding: utf-8 -*-

countries=(
(u"DE", u"DEU", u"276", u"Deutschland", u"Federal Republic of Germany"),
(u"AT", u"AUT", u"040", u"Österreich", u"Republic of Austria"),
(u"CH", u"CHE", u"756", u"Schweiz", u"Swiss Confederation"),
(u"AF", u"AFG", u"004", u"Afghanistan", u"Islamic Republic of Afghanistan"),
(u"AX", u"ALA", u"248", u"Åland Islands"),
(u"AL", u"ALB", u"008", u"Albania", u"Republic of Albania"),
(u"DZ", u"DZA", u"012", u"Algeria", u"People's Democratic Republic of Algeria"),
(u"AS", u"ASM", u"016", u"American Samoa"),
(u"AD", u"AND", u"020", u"Andorra", u"Principality of Andorra"),
(u"AO", u"AGO", u"024", u"Angola", u"Republic of Angola"),
(u"AI", u"AIA", u"660", u"Anguilla"),
(u"AQ", u"ATA", u"010", u"Antarctica"),
(u"AG", u"ATG", u"028", u"Antigua and Barbuda"),
(u"AR", u"ARG", u"032", u"Argentina", u"Argentine Republic"),
(u"AM", u"ARM", u"051", u"Armenia", u"Republic of Armenia"),
(u"AW", u"ABW", u"533", u"Aruba"),
(u"AU", u"AUS", u"036", u"Australia"),
(u"AZ", u"AZE", u"031", u"Azerbaijan", u"Republic of Azerbaijan"),
(u"BS", u"BHS", u"044", u"Bahamas", u"Commonwealth of the Bahamas"),
(u"BH", u"BHR", u"048", u"Bahrain", u"Kingdom of Bahrain"),
(u"BD", u"BGD", u"050", u"Bangladesh", u"People's Republic of Bangladesh"),
(u"BB", u"BRB", u"052", u"Barbados"),
(u"BY", u"BLR", u"112", u"Belarus", u"Republic of Belarus"),
(u"BE", u"BEL", u"056", u"Belgium", u"Kingdom of Belgium"),
(u"BZ", u"BLZ", u"084", u"Belize"),
(u"BJ", u"BEN", u"204", u"Benin", u"Republic of Benin"),
(u"BM", u"BMU", u"060", u"Bermuda"),
(u"BT", u"BTN", u"064", u"Bhutan", u"Kingdom of Bhutan"),
(u"BO", u"BOL", u"068", u"Bolivia", u"Republic of Bolivia"),
(u"BA", u"BIH", u"070", u"Bosnia and Herzegovina", u"Republic of Bosnia and Herzegovina"),
(u"BW", u"BWA", u"072", u"Botswana", u"Republic of Botswana"),
(u"BV", u"BVT", u"074", u"Bouvet Island"),
(u"BR", u"BRA", u"076", u"Brazil", u"Federative Republic of Brazil"),
(u"IO", u"IOT", u"086", u"British Indian Ocean Territory"),
(u"BN", u"BRN", u"096", u"Brunei Darussalam"),
(u"BG", u"BGR", u"100", u"Bulgaria", u"Republic of Bulgaria"),
(u"BF", u"BFA", u"854", u"Burkina Faso"),
(u"BI", u"BDI", u"108", u"Burundi", u"Republic of Burundi"),
(u"KH", u"KHM", u"116", u"Cambodia", u"Kingdom of Cambodia"),
(u"CM", u"CMR", u"120", u"Cameroon", u"Republic of Cameroon"),
(u"CA", u"CAN", u"124", u"Canada"),
(u"CV", u"CPV", u"132", u"Cape Verde", u"Republic of Cape Verde"),
(u"KY", u"CYM", u"136", u"Cayman Islands"),
(u"CF", u"CAF", u"140", u"Central African Republic"),
(u"TD", u"TCD", u"148", u"Chad", u"Republic of Chad"),
(u"CL", u"CHL", u"152", u"Chile", u"Republic of Chile"),
(u"CN", u"CHN", u"156", u"China", u"People's Republic of China"),
(u"CX", u"CXR", u"162", u"Christmas Island"),
(u"CC", u"CCK", u"166", u"Cocos (Keeling) Islands"),
(u"CO", u"COL", u"170", u"Colombia", u"Republic of Colombia"),
(u"KM", u"COM", u"174", u"Comoros", u"Union of the Comoros"),
(u"CG", u"COG", u"178", u"Congo", u"Republic of the Congo"),
(u"CD", u"COD", u"180", u"Congo, The Democratic Republic of the"),
(u"CK", u"COK", u"184", u"Cook Islands"),
(u"CR", u"CRI", u"188", u"Costa Rica", u"Republic of Costa Rica"),
(u"CI", u"CIV", u"384", u"Côte d'Ivoire", u"Republic of Côte d'Ivoire"),
(u"HR", u"HRV", u"191", u"Croatia", u"Republic of Croatia"),
(u"CU", u"CUB", u"192", u"Cuba", u"Republic of Cuba"),
(u"CY", u"CYP", u"196", u"Cyprus", u"Republic of Cyprus"),
(u"CZ", u"CZE", u"203", u"Czech Republic"),
(u"DK", u"DNK", u"208", u"Denmark", u"Kingdom of Denmark"),
(u"DJ", u"DJI", u"262", u"Djibouti", u"Republic of Djibouti"),
(u"DM", u"DMA", u"212", u"Dominica", u"Commonwealth of Dominica"),
(u"DO", u"DOM", u"214", u"Dominican Republic"),
(u"EC", u"ECU", u"218", u"Ecuador", u"Republic of Ecuador"),
(u"EG", u"EGY", u"818", u"Egypt", u"Arab Republic of Egypt"),
(u"SV", u"SLV", u"222", u"El Salvador", u"Republic of El Salvador"),
(u"GQ", u"GNQ", u"226", u"Equatorial Guinea", u"Republic of Equatorial Guinea"),
(u"ER", u"ERI", u"232", u"Eritrea"),
(u"EE", u"EST", u"233", u"Estonia", u"Republic of Estonia"),
(u"ET", u"ETH", u"231", u"Ethiopia", u"Federal Democratic Republic of Ethiopia"),
(u"FK", u"FLK", u"238", u"Falkland Islands (Malvinas)"),
(u"FO", u"FRO", u"234", u"Faroe Islands"),
(u"FJ", u"FJI", u"242", u"Fiji", u"Republic of the Fiji Islands"),
(u"FI", u"FIN", u"246", u"Finland", u"Republic of Finland"),
(u"FR", u"FRA", u"250", u"France", u"French Republic"),
(u"GF", u"GUF", u"254", u"French Guiana"),
(u"PF", u"PYF", u"258", u"French Polynesia"),
(u"TF", u"ATF", u"260", u"French Southern Territories"),
(u"GA", u"GAB", u"266", u"Gabon", u"Gabonese Republic"),
(u"GM", u"GMB", u"270", u"Gambia", u"Republic of the Gambia"),
(u"GE", u"GEO", u"268", u"Georgia"),
(u"GH", u"GHA", u"288", u"Ghana", u"Republic of Ghana"),
(u"GI", u"GIB", u"292", u"Gibraltar"),
(u"GR", u"GRC", u"300", u"Greece", u"Hellenic Republic"),
(u"GL", u"GRL", u"304", u"Greenland"),
(u"GD", u"GRD", u"308", u"Grenada"),
(u"GP", u"GLP", u"312", u"Guadeloupe"),
(u"GU", u"GUM", u"316", u"Guam"),
(u"GT", u"GTM", u"320", u"Guatemala", u"Republic of Guatemala"),
(u"GG", u"GGY", u"831", u"Guernsey"),
(u"GN", u"GIN", u"324", u"Guinea", u"Republic of Guinea"),
(u"GW", u"GNB", u"624", u"Guinea-Bissau", u"Republic of Guinea-Bissau"),
(u"GY", u"GUY", u"328", u"Guyana", u"Republic of Guyana"),
(u"HT", u"HTI", u"332", u"Haiti", u"Republic of Haiti"),
(u"HM", u"HMD", u"334", u"Heard Island and McDonald Islands"),
(u"VA", u"VAT", u"336", u"Holy See (Vatican City State)"),
(u"HN", u"HND", u"340", u"Honduras", u"Republic of Honduras"),
(u"HK", u"HKG", u"344", u"Hong Kong", u"Hong Kong Special Administrative Region of China"),
(u"HU", u"HUN", u"348", u"Hungary", u"Republic of Hungary"),
(u"IS", u"ISL", u"352", u"Iceland", u"Republic of Iceland"),
(u"IN", u"IND", u"356", u"India", u"Republic of India"),
(u"ID", u"IDN", u"360", u"Indonesia", u"Republic of Indonesia"),
(u"IR", u"IRN", u"364", u"Iran, Islamic Republic of", u"Islamic Republic of Iran"),
(u"IQ", u"IRQ", u"368", u"Iraq", u"Republic of Iraq"),
(u"IE", u"IRL", u"372", u"Ireland"),
(u"IM", u"IMN", u"833", u"Isle of Man"),
(u"IL", u"ISR", u"376", u"Israel", u"State of Israel"),
(u"IT", u"ITA", u"380", u"Italy", u"Italian Republic"),
(u"JM", u"JAM", u"388", u"Jamaica"),
(u"JP", u"JPN", u"392", u"Japan"),
(u"JE", u"JEY", u"832", u"Jersey"),
(u"JO", u"JOR", u"400", u"Jordan", u"Hashemite Kingdom of Jordan"),
(u"KZ", u"KAZ", u"398", u"Kazakhstan", u"Republic of Kazakhstan"),
(u"KE", u"KEN", u"404", u"Kenya", u"Republic of Kenya"),
(u"KI", u"KIR", u"296", u"Kiribati", u"Republic of Kiribati"),
(u"KP", u"PRK", u"408", u"Korea, Democratic People's Republic of", u"Democratic People's Republic of Korea"),
(u"KR", u"KOR", u"410", u"Korea, Republic of"),
(u"KW", u"KWT", u"414", u"Kuwait", u"State of Kuwait"),
(u"KG", u"KGZ", u"417", u"Kyrgyzstan", u"Kyrgyz Republic"),
(u"LA", u"LAO", u"418", u"Lao People's Democratic Republic"),
(u"LV", u"LVA", u"428", u"Latvia", u"Republic of Latvia"),
(u"LB", u"LBN", u"422", u"Lebanon", u"Lebanese Republic"),
(u"LS", u"LSO", u"426", u"Lesotho", u"Kingdom of Lesotho"),
(u"LR", u"LBR", u"430", u"Liberia", u"Republic of Liberia"),
(u"LY", u"LBY", u"434", u"Libyan Arab Jamahiriya", u"Socialist People's Libyan Arab Jamahiriya"),
(u"LI", u"LIE", u"438", u"Liechtenstein", u"Principality of Liechtenstein"),
(u"LT", u"LTU", u"440", u"Lithuania", u"Republic of Lithuania"),
(u"LU", u"LUX", u"442", u"Luxembourg", u"Grand Duchy of Luxembourg"),
(u"MO", u"MAC", u"446", u"Macao", u"Macao Special Administrative Region of China"),
(u"MK", u"MKD", u"807", u"Macedonia, Republic of", u"The Former Yugoslav Republic of Macedonia"),
(u"MG", u"MDG", u"450", u"Madagascar", u"Republic of Madagascar"),
(u"MW", u"MWI", u"454", u"Malawi", u"Republic of Malawi"),
(u"MY", u"MYS", u"458", u"Malaysia"),
(u"MV", u"MDV", u"462", u"Maldives", u"Republic of Maldives"),
(u"ML", u"MLI", u"466", u"Mali", u"Republic of Mali"),
(u"MT", u"MLT", u"470", u"Malta", u"Republic of Malta"),
(u"MH", u"MHL", u"584", u"Marshall Islands", u"Republic of the Marshall Islands"),
(u"MQ", u"MTQ", u"474", u"Martinique"),
(u"MR", u"MRT", u"478", u"Mauritania", u"Islamic Republic of Mauritania"),
(u"MU", u"MUS", u"480", u"Mauritius", u"Republic of Mauritius"),
(u"YT", u"MYT", u"175", u"Mayotte"),
(u"MX", u"MEX", u"484", u"Mexico", u"United Mexican States"),
(u"FM", u"FSM", u"583", u"Micronesia, Federated States of", u"Federated States of Micronesia"),
(u"MD", u"MDA", u"498", u"Moldova", u"Republic of Moldova"),
(u"MC", u"MCO", u"492", u"Monaco", u"Principality of Monaco"),
(u"MN", u"MNG", u"496", u"Mongolia"),
(u"ME", u"MNE", u"499", u"Montenegro", u"Montenegro"),
(u"MS", u"MSR", u"500", u"Montserrat"),
(u"MA", u"MAR", u"504", u"Morocco", u"Kingdom of Morocco"),
(u"MZ", u"MOZ", u"508", u"Mozambique", u"Republic of Mozambique"),
(u"MM", u"MMR", u"104", u"Myanmar", u"Union of Myanmar"),
(u"NA", u"NAM", u"516", u"Namibia", u"Republic of Namibia"),
(u"NR", u"NRU", u"520", u"Nauru", u"Republic of Nauru"),
(u"NP", u"NPL", u"524", u"Nepal", u"Federal Democratic Republic of Nepal"),
(u"NL", u"NLD", u"528", u"Netherlands", u"Kingdom of the Netherlands"),
(u"AN", u"ANT", u"530", u"Netherlands Antilles"),
(u"NC", u"NCL", u"540", u"New Caledonia"),
(u"NZ", u"NZL", u"554", u"New Zealand"),
(u"NI", u"NIC", u"558", u"Nicaragua", u"Republic of Nicaragua"),
(u"NE", u"NER", u"562", u"Niger", u"Republic of the Niger"),
(u"NG", u"NGA", u"566", u"Nigeria", u"Federal Republic of Nigeria"),
(u"NU", u"NIU", u"570", u"Niue", u"Republic of Niue"),
(u"NF", u"NFK", u"574", u"Norfolk Island"),
(u"MP", u"MNP", u"580", u"Northern Mariana Islands", u"Commonwealth of the Northern Mariana Islands"),
(u"NO", u"NOR", u"578", u"Norway", u"Kingdom of Norway"),
(u"OM", u"OMN", u"512", u"Oman", u"Sultanate of Oman"),
(u"PK", u"PAK", u"586", u"Pakistan", u"Islamic Republic of Pakistan"),
(u"PW", u"PLW", u"585", u"Palau", u"Republic of Palau"),
(u"PS", u"PSE", u"275", u"Palestinian Territory, Occupied", u"Occupied Palestinian Territory"),
(u"PA", u"PAN", u"591", u"Panama", u"Republic of Panama"),
(u"PG", u"PNG", u"598", u"Papua New Guinea"),
(u"PY", u"PRY", u"600", u"Paraguay", u"Republic of Paraguay"),
(u"PE", u"PER", u"604", u"Peru", u"Republic of Peru"),
(u"PH", u"PHL", u"608", u"Philippines", u"Republic of the Philippines"),
(u"PN", u"PCN", u"612", u"Pitcairn"),
(u"PL", u"POL", u"616", u"Poland", u"Republic of Poland"),
(u"PT", u"PRT", u"620", u"Portugal", u"Portuguese Republic"),
(u"PR", u"PRI", u"630", u"Puerto Rico"),
(u"QA", u"QAT", u"634", u"Qatar", u"State of Qatar"),
(u"RE", u"REU", u"638", u"Reunion"),
(u"RO", u"ROU", u"642", u"Romania"),
(u"RU", u"RUS", u"643", u"Russian Federation"),
(u"RW", u"RWA", u"646", u"Rwanda", u"Rwandese Republic"),
(u"BL", u"BLM", u"652", u"Saint Barthélemy"),
(u"SH", u"SHN", u"654", u"Saint Helena"),
(u"KN", u"KNA", u"659", u"Saint Kitts and Nevis"),
(u"LC", u"LCA", u"662", u"Saint Lucia"),
(u"MF", u"MAF", u"663", u"Saint Martin (French part)"),
(u"PM", u"SPM", u"666", u"Saint Pierre and Miquelon"),
(u"VC", u"VCT", u"670", u"Saint Vincent and the Grenadines"),
(u"WS", u"WSM", u"882", u"Samoa", u"Independent State of Samoa"),
(u"SM", u"SMR", u"674", u"San Marino", u"Republic of San Marino"),
(u"ST", u"STP", u"678", u"Sao Tome and Principe", u"Democratic Republic of Sao Tome and Principe"),
(u"SA", u"SAU", u"682", u"Saudi Arabia", u"Kingdom of Saudi Arabia"),
(u"SN", u"SEN", u"686", u"Senegal", u"Republic of Senegal"),
(u"RS", u"SRB", u"688", u"Serbia", u"Republic of Serbia"),
(u"SC", u"SYC", u"690", u"Seychelles", u"Republic of Seychelles"),
(u"SL", u"SLE", u"694", u"Sierra Leone", u"Republic of Sierra Leone"),
(u"SG", u"SGP", u"702", u"Singapore", u"Republic of Singapore"),
(u"SK", u"SVK", u"703", u"Slovakia", u"Slovak Republic"),
(u"SI", u"SVN", u"705", u"Slovenia", u"Republic of Slovenia"),
(u"SB", u"SLB", u"090", u"Solomon Islands"),
(u"SO", u"SOM", u"706", u"Somalia", u"Somali Republic"),
(u"ZA", u"ZAF", u"710", u"South Africa", u"Republic of South Africa"),
(u"GS", u"SGS", u"239", u"South Georgia and the South Sandwich Islands"),
(u"ES", u"ESP", u"724", u"Spain", u"Kingdom of Spain"),
(u"LK", u"LKA", u"144", u"Sri Lanka", u"Democratic Socialist Republic of Sri Lanka"),
(u"SD", u"SDN", u"736", u"Sudan", u"Republic of the Sudan"),
(u"SR", u"SUR", u"740", u"Suriname", u"Republic of Suriname"),
(u"SJ", u"SJM", u"744", u"Svalbard and Jan Mayen"),
(u"SZ", u"SWZ", u"748", u"Swaziland", u"Kingdom of Swaziland"),
(u"SE", u"SWE", u"752", u"Sweden", u"Kingdom of Sweden"),
(u"SY", u"SYR", u"760", u"Syrian Arab Republic"),
(u"TW", u"TWN", u"158", u"Taiwan", u"Taiwan, Province of China", u"Taiwan, Province of China"),
(u"TJ", u"TJK", u"762", u"Tajikistan", u"Republic of Tajikistan"),
(u"TZ", u"TZA", u"834", u"Tanzania, United Republic of", u"United Republic of Tanzania"),
(u"TH", u"THA", u"764", u"Thailand", u"Kingdom of Thailand"),
(u"TL", u"TLS", u"626", u"Timor-Leste", u"Democratic Republic of Timor-Leste"),
(u"TG", u"TGO", u"768", u"Togo", u"Togolese Republic"),
(u"TK", u"TKL", u"772", u"Tokelau"),
(u"TO", u"TON", u"776", u"Tonga", u"Kingdom of Tonga"),
(u"TT", u"TTO", u"780", u"Trinidad and Tobago", u"Republic of Trinidad and Tobago"),
(u"TN", u"TUN", u"788", u"Tunisia", u"Republic of Tunisia"),
(u"TR", u"TUR", u"792", u"Turkey", u"Republic of Turkey"),
(u"TM", u"TKM", u"795", u"Turkmenistan"),
(u"TC", u"TCA", u"796", u"Turks and Caicos Islands"),
(u"TV", u"TUV", u"798", u"Tuvalu"),
(u"UG", u"UGA", u"800", u"Uganda", u"Republic of Uganda"),
(u"UA", u"UKR", u"804", u"Ukraine"),
(u"AE", u"ARE", u"784", u"United Arab Emirates"),
(u"GB", u"GBR", u"826", u"United Kingdom", u"United Kingdom of Great Britain and Northern Ireland"),
(u"US", u"USA", u"840", u"United States", u"United States of America"),
(u"UM", u"UMI", u"581", u"United States Minor Outlying Islands"),
(u"UY", u"URY", u"858", u"Uruguay", u"Eastern Republic of Uruguay"),
(u"UZ", u"UZB", u"860", u"Uzbekistan", u"Republic of Uzbekistan"),
(u"VU", u"VUT", u"548", u"Vanuatu", u"Republic of Vanuatu"),
(u"VE", u"VEN", u"862", u"Venezuela", u"Bolivarian Republic of Venezuela"),
(u"VN", u"VNM", u"704", u"Viet Nam", u"Socialist Republic of Viet Nam"),
(u"VG", u"VGB", u"092", u"Virgin Islands, British", u"British Virgin Islands"),
(u"VI", u"VIR", u"850", u"Virgin Islands, U.S.", u"Virgin Islands of the United States"),
(u"WF", u"WLF", u"876", u"Wallis and Futuna"),
(u"EH", u"ESH", u"732", u"Western Sahara"),
(u"YE", u"YEM", u"887", u"Yemen", u"Republic of Yemen"),
(u"ZM", u"ZMB", u"894", u"Zambia", u"Republic of Zambia"),
(u"ZW", u"ZWE", u"716", u"Zimbabwe", u"Republic of Zimbabwe"),
)
_cl = []

def GetConf(code):
    """
    Load country by code
    """
    code = code.upper()
    if len(code)==2:
        for c in countries:
            if c[0] == code:
                return _GetDict(c)
    if len(code)==3:
        for c in countries:
            if c[1] == code:
                return _GetDict(c)
    return {}


def GetCountries():
    """
    load codelist of countries and cache
    """
    if len(_cl) == 0:
        for c in countries:
            _cl.append({"id": c[1], "name": c[3]})
    return _cl


def _GetDict(c):
    d = {"code": c[1], "code2": c[0], "numeric": c[2], "name": c[3]}
    if len(c) > 4:
        d["name2"] = c[4]
    return d


#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------
__doc__ = """
"""

import hashlib
from datetime import datetime

from nive.i18n import _
from nive.definitions import implements, IUser

from nive.components.objects.base import ObjectBase



def Sha(password):
    return hashlib.sha224(password).hexdigest()


class user(ObjectBase):
    """
    User object with groups and login possibility. 
    """
    implements(IUser)
    
    @property
    def identity(self):
        return self.data.get(self.parent.identityField,str(self.id))

    def __str__(self):
        return str(self.identity)
    
    def Init(self):
        self.groups = tuple(self.data.get("groups",()))
        self.ListenEvent("commit", "OnCommit")


    def Authenticate(self, password):
        return Sha(password) == self.data["password"]

    
    def Login(self):
        """
        events: login(lastlogin)
        """
        lastlogin = self.data.get("lastlogin")
        date = datetime.now()
        self.data.set("lastlogin", date)
        self.Commit(self)
        self.Signal("login", lastlogin=lastlogin)


    def Logout(self):
        """
        events: logout()
        """
        self.Signal("logout")
        self.Commit(self)


    def OnCommit(self):
        self.HashPassword()
        t = self.ReadableName()
        if t != self.meta["title"]:
            self.meta["title"] = t


    def HashPassword(self):
        if not self.data.HasTempKey("password"):
            return
        pw = Sha(self.data.password)
        self.data["password"] = pw


    def ReadableName(self):
        if self.data.surname or self.data.lastname: 
            return u" ".join([self.data.surname, self.data.lastname])
        return self.data.name


    def SecureUpdate(self, data, user):
        """
        Update existing user data.
        name, groups, pool_state cannot be changed
        """
        if data.has_key("name"):
            del data["name"]
        if data.has_key("groups"):
            del data["groups"]
        if data.has_key("pool_state"):
            del data["pool_state"]

        if not self.Update(data, user):
            return False, [_(u"Update failed.")]
        
        self.Commit(user)
        return True, []


    def UpdateGroups(self, groups):
        """
        update groups of user
        """
        self.groups = tuple(groups)
        self.data["groups"] = groups
        return True


    def AddGroup(self, group, user):
        """
        add user to this group
        
        event: securityCahnged()
        """
        if group in self.groups:
            return True
        g = list(self.groups)
        g.append(group)
        self.groups = tuple(g)
        self.data["groups"] = g
        self.Commit(user)
        return True


    def GetGroups(self, context=None):
        """
        Returns the users gloabal groups as tuple.
        Local assignments are not supported, `context` is currently unused.
        """
        return self.groups


    def InGroups(self, groups):
        """
        check if user has one of these groups
        """
        if isinstance(groups, basestring):
            return groups in self.groups
        for g in groups:
            if g in self.groups:
                return True
        return False



# user definition ------------------------------------------------------------------
from nive.definitions import StagUser, ObjectConf, FieldConf
#@nive_module
configuration = ObjectConf(
    id = "user",
    name = _(u"User"),
    dbparam = "users",
    context = "nive.userdb.user.user",
    template = "user.pt",
    selectTag = StagUser,
    container = False,
    description = __doc__
)

configuration.data = (
    FieldConf(id="name",     datatype="string",      size= 30, default=u"", required=1, name=_(u"User ID"), description=u""),
    FieldConf(id="password", datatype="password",    size=100, default=u"", required=1, name=_(u"Password"), description=u""),
    FieldConf(id="email",    datatype="email",       size=255, default=u"", required=1, name=_(u"Email"), description=u""),
    FieldConf(id="groups",   datatype="mcheckboxes", size=255, default=u"", name=_(u"Groups"), settings={"codelist":"groups"}, description=u""),
    
    FieldConf(id="notify",   datatype="bool",        size= 4,  default=True, name=_(u"Activate email notifications"), description=u""),
    
    FieldConf(id="surname",  datatype="string",      size=100, default=u"", name=_(u"Surname"), description=u""),
    FieldConf(id="lastname", datatype="string",      size=100, default=u"", name=_(u"Lastname"), description=u""),
    FieldConf(id="organisation", datatype="string",  size=255, default=u"", name=_(u"Organisation"), description=u""),
    
    FieldConf(id="lastlogin", datatype="datetime", size=0, readonly=True, default=u"", name=_(u"Last login"), description=u""),
    
)

#password2 = FieldConf(id="password2", datatype="password", size= 30, default="", required=1, name="Passwort - Wiederholung", description="")
configuration.forms = {
    "create": {"fields": ["name", "password", "email", "surname", "lastname"]},
    "edit":   {"fields": ["password", "email", "surname", "lastname"]},
}




import sys
WIN = sys.platform == "win32"

# sqlite and mysql
if WIN:
    ROOT = "c:\\Temp\\nive\\"
else:
    ROOT = "/var/tmp/nive/"

# mysql
HOST = "localhost"
PORT = ""
USER = "root"
DATABASE = "ut_nive"
PASSWORD = ""



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = ""

import types


from nive.tools import Tool, ToolView
from nive.definitions import ToolConf, ViewConf, FieldConf, IApplication, Structure, MetaTbl
from nive.i18n import _
from nive.views import BaseView
from nive.helper import FakeLocalizer
from nive.utils.dataPool2.base import OperationalError

from pyramid.i18n import get_localizer
from pyramid.threadlocal import get_current_request

   
   
class dbView(BaseView):
    """
    """
    
    def view(self):
        """
        Run a tool by rendering the default form and execute on submit.
        This function does not return a valid Response object. This view is meant to
        be called from another view or template:
        ``view.RenderView(tool)``
        """
        tool = self.context
        values = self.GetFormValues(method="POST")
        values["request"] = self.request
        result = tool.Run(**values)
        data = tool.stream
        if not isinstance(data, basestring):
            try:
                data = data.getvalue()
            except:
                data = str(data)
        return self.SendResponse(data, mime=self.context.mimetype, raiseException=False) 
    
            
configuration = ToolConf()
configuration.id = "dbStructureUpdater"
configuration.context = "nive.components.tools.dbStructureUpdater.dbStructureUpdater"
configuration.name = _(u"Database Structure")
configuration.description = _(u"Generate or update the database structure based on configuration settings.")
configuration.apply = (IApplication,)
configuration.data = [
    FieldConf(id="modify",     datatype="bool", default=0, name=_(u"Modify existing columns"),  description=_(u"Change existing database columns to new configuration. Depending on the changes, data may be lost!")),
    FieldConf(id="showSystem", datatype="bool", default=0, name=_(u"Show system columns"),      description=u"")
]
configuration.mimetype = "text/html"
configuration.views = [
    ViewConf(name="", view=dbView, attr="view", permission="system", context="nive.components.tools.dbStructureUpdater.dbStructureUpdater")
]

class dbStructureUpdater(Tool):

    def _Run(self, **values):

        result = 1
        importWf = 1
        importSecurity = 0
        showSystem = values.get("showSystem")
        modify = values.get("modify")
        request = values["original"]
        
        try:
            localizer = get_localizer(get_current_request())
        except:
            localizer = FakeLocalizer()
        #localizer.translate(term) 

        text = _(u""" <div class="well">
This tool compares the physically existing database structure (tables, columns) with the current configuration settings.
The database structure is shown on the left, configuration settings on the right. <br/><br/>
Existing database columns will only be altered if manually selected in the 'Modify' column. Modifying a table may destroy the data
stored (e.g if converted from string to integer), so don't forget to create backups of the database before modifying anything.<br/>
By default this tool will only create new tables and columns and never delete any column.
 </div>       """)
        self.stream.write(localizer.translate(_(text)))

        self.stream.write(u"""<form action="" method="post">
                     <input type="hidden" name="tag" value="dbStructureUpdater">
                     <input type="hidden" name="modify" value="1">""")
        app = self.app
        try:
            conf = app.dbConfiguration
            connection = app.NewConnection()
            if not connection:
                self.stream.write(localizer.translate(_(u"""<div class="alert alert-error">No database connection configured</div>""")))
                return 0
        except OperationalError, e:
            self.stream.write(localizer.translate(_(u"""<div class="alert alert-error">No database connection configured</div>""")))
            return 0

        db = connection.GetDBManager()
        self.stream.write(localizer.translate(_(u"<h4>Database '${name}' ${host} </h4><br/>", mapping={"host":conf.host, "name":conf.dbName})))

        if not db:
            self.stream.write(localizer.translate(_(u"<div class='alert alert-error'>Database connection error (${name})</div>", mapping={"name": app.dbConfiguration.context})))
            return 0
        
        # check database exists
        if not db.IsDatabase(conf.get("dbName")):
            db.CreateDatabase(conf.get("dbName"))
            self.stream.write(u"")
            self.stream.write(u"")
            self.stream.write(localizer.translate(_(u"<div class='alert alert-success'>Database created: '${name}'</div>", mapping={"name": conf.dbName})))
            db.dbConn.commit()
        db.UseDatabase(conf.get("dbName"))
        
        
        # check types for data tables -------------------------------------------------------------
        aTypes = app.GetAllObjectConfs()
        
        for aT in aTypes:
            fmt = aT["data"]
            if(fmt == []):
                continue

            m = None
            if modify:
                m = request.get(aT["dbparam"])
                if isinstance(m, basestring):
                    m = [m]
            if not db.UpdateStructure(aT["dbparam"], fmt, m):
                self.stream.write(u"")
                self.stream.write(localizer.translate(_(u"<div class='alert alert-error'>Table update failed: ${dbparam} (type: ${name})</div>", mapping=aT)))
                result = 0
                continue
            db.dbConn.commit()
                
            self.printStructure(db.GetColumns(aT["dbparam"], fmt), aT["dbparam"], fmt, db, localizer)

        # check meta table exists and update ---------------------------------------------------------------
        meta = app.GetAllMetaFlds(ignoreSystem=False)
        tableName = MetaTbl

        if not db.IsTable(tableName):
            if not db.CreateTable(tableName, columns=meta):
                self.stream.write(localizer.translate(_(u"<div class='alert alert-error'>Table creation failed (pool_meta)</div>")))
                return 0
            db.dbConn.commit()

        # create and check modified fields
        m = None
        if modify:
            m = request.get(tableName)
            if isinstance(m, basestring):
                m = [m]
        if not db.UpdateStructure(tableName, meta, m):
            self.stream.write(localizer.translate(_(u"<div class='alert alert-error'>Table update failed (pool_meta)</div>")))
            result = 0
        db.dbConn.commit()
        
        self.printStructure(db.GetColumns(tableName, meta), tableName, meta, db, localizer)


        # check structure tables exist and update ------------------------------------------------------------
        for table in Structure.items():
            tableName = table[0]
            fields = table[1]["fields"]
            identity = table[1]["identity"]
            if not db.IsTable(tableName):
                if not db.CreateTable(tableName, columns=fields, createIdentity = bool(identity), primaryKeyName = identity):
                    self.stream.write(localizer.translate(_(u"<div class='alert alert-error'>Table creation failed (${name})</div>",mapping={"name":tableName})))
                    return 0
                db.dbConn.commit()
        
            # create and check modified fields
            m = None
            if modify:
                m = request.get(tableName)
                if isinstance(m, basestring):
                    m = [m]
            if not db.UpdateStructure(tableName, fields, m, createIdentity = bool(identity)):
                self.stream.write(localizer.translate(_(u"<div class='alert alert-error'>Table creation failed (${name})</div>",mapping={"name":tableName})))
                result = 0
            db.dbConn.commit()

            if showSystem:
                self.printStructure(db.GetColumns(tableName, fields), tableName, fields, db, localizer)



        if db.modifyColumns:
            self.stream.write(u"""<div class="alert alert-error"><input class="btn" type="submit" name="submit" value=" %s "/><br/>%s</div>""" % (
                localizer.translate(_(u"Modify selected columns")), 
                localizer.translate(_(u"Changes on existing columns have to be applied manually. This will write selected 'Configuration settings' to the database.<br/> <b>Warning: This may destroy something!</b>"))))
        self.stream.write(localizer.translate(u"</form>"))
        connection.close()
        return result

    
    def printStructure(self, structure, table, fmt, db, localizer):
        header = u"""
<h4>%(Table)s: %(tablename)s</h4>
<table class="table"><tbody>
<tr><td colspan="5">%(Db Columns)s</td>                                        <td colspan="2">%(Configuration settings)s</td></tr>
<tr><td>%(ID)s</td><td>%(Type)s</td><td>%(Default)s</td><td>%(Settings)s</td>  <td>%(Modify?)s</td><td></td></tr>
"""  % {"tablename":table, 
        "Table": localizer.translate(_(u"Table")), 
        "Db Columns": localizer.translate(_(u"Database settings")), 
        "Configuration settings": localizer.translate(_(u"Configuration settings")),
        "ID":localizer.translate(_(u"ID")),
        "Type":localizer.translate(_(u"Type")),
        "Default":localizer.translate(_(u"Default")),
        "Settings":localizer.translate(_(u"Settings")),
        "Modify?":localizer.translate(_(u"Modify?"))
       }

        row = u"""
<tr><td>%(id)s</td><td>%(type)s</td><td>%(default)s</td><td>Not null: %(null)s, Identity: %(identity)s</td>        <td>%(Modify)s</td><td>%(Conf)s</td></tr>
"""

        cb = u"""
<input type="checkbox" name="%s" value="%s">""" 

        footer = u"""
</tbody></table> <br/>"""


        self.stream.write(header)
        for col in structure:
            id = col
            col = structure[col].get("db")
            if not col:
                col = {"id":id, "type": u"", "default": u"", "null": u"", "identity": u""}
            conf = u""
            for d in fmt:
                if col and d["id"].upper() == col["id"].upper():
                    conf = db.ConvertConfToColumnOptions(d)
                    break
            col["Modify"] = cb % (table, id)
            col["Conf"] = conf
            self.stream.write(row % col)

        self.stream.write(footer)

        return 
    
    



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Code
-----
Text element to add code snippets to web pages. Supports Javascript, raw HTML and CSS. 
"""

from nive.i18n import _
from nive.definitions import StagPageElement, ObjectConf, FieldConf
from nive.components.objects.base import PageElementBase


class code(PageElementBase):
    """
    """
    



# text type definition ------------------------------------------------------------------
#@nive_module
configuration = ObjectConf(
    id = "code",
    name = _(u"Code"),
    dbparam = "codes",
    context = "nive.cms.code.code",
    template = "code.pt",
    selectTag = StagPageElement,
    icon = "nive.cms.cmsview:static/images/types/code.png",
    description = _(u"Text element to add code snippets to web pages. Supports Javascript, HTML and CSS.")
)

ct =[ {'id': u'raw', 'name': _(u'No format')},
      {'id': u'js', 'name': _(u'Javascript')},
      {'id': u'html', 'name': _(u'HTML')},
      {'id': u'css', 'name': _(u'CSS')},
]

configuration.data = [
    FieldConf(id="codetype",  datatype="list", size=10, default="raw", name=_(u"Type"), listItems=ct, fulltext=False, description=u""),
    FieldConf(id="codeblock", datatype="code", size=50000, default="", name=_(u"Code"), fulltext=False, description=u""),
]

fields = ["title","codeblock", "pool_groups"]
configuration.forms = {"create": {"fields":fields}, "edit": {"fields":fields}}

configuration.views = []



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------
"""
Data Pool Sqlite Module
----------------------
*Requires python-sqlite*
"""

import threading
import sqlite3
from time import time

from nive.utils.utils import STACKF

from nive.utils.dataPool2.base import Base, Entry
from nive.utils.dataPool2.base import NotFound
from nive.utils.dataPool2.connection import Connection, ConnectionThreadLocal, ConnectionRequest

from nive.utils.dataPool2.files import FileManager, FileEntry
from nive.utils.dataPool2.dbManager import Sqlite3Manager

from nive.definitions import OperationalError



class Sqlite3Connection(Connection):
    """
    Sqlite connection handling class

    config parameter in dictionary:
    db = database path
    
    user = unused - database user
    password = unused - password user
    host = unused - host server
    port = unused - port server

    timeout is set to 3.
    """

    def __init__(self, config = None, connectNow = True):
        self.db = None
        self.configuration = config
        
        self.placeholder = u"?"
        self.check_same_thread = False
        if connectNow:
            self.connect()
        

    def ping(self):
        """ ping database server """
        return True


    def connect(self):
        """ Close and connect to server """
        #t = time()
        conf = self.configuration
        if not conf.dbName:
            raise OperationalError, "Connection failed. Database name is empty." 
        db = sqlite3.connect(conf.dbName, check_same_thread=self.check_same_thread)
        if not db:
            raise OperationalError, "Cannot connect to database '%s'" % (conf.dbName)
        c = db.cursor()
        c.execute("PRAGMA journal_mode = TRUNCATE")
        #c.execute("PRAGMA secure_delete = 0")
        #c.execute("PRAGMA temp_store = MEMORY")
        c.execute("PRAGMA synchronous = OFF")
        c.close()
        self._set(db)
        #print "connect:", time() - t
        return db


    def IsConnected(self):
        """ Check if database is connected """
        try:
            db = self._get()
            return db.cursor()!=None
        except:
            return False
    

    def GetDBManager(self):
        """ returns the database manager obj """
        aDB = Sqlite3Manager()
        aDB.SetDB(self.PrivateConnection())
        return aDB


    def FmtParam(self, param):
        """ format a parameter for sql queries like literal for  db"""
        if isinstance(param, (int, long, float)):
            return unicode(param)
        d = unicode(param)
        if d.find(u'"')!=-1:
            d = d.replace(u'"',u'\\"')
        return u'"%s"'%d


    def PrivateConnection(self):
        """ This function will return a new and raw connection. It is up to the caller to close this connection. """
        conf = self.configuration
        if not conf.dbName:
            raise OperationalError, "Connection failed. Database name is empty." 
        db = sqlite3.connect(conf.dbName, check_same_thread=self.check_same_thread)
        return db



class Sqlite3ConnThreadLocal(Sqlite3Connection, ConnectionThreadLocal):
    """
    Caches database connections as thread local values.
    """

    def __init__(self, config = None, connectNow = True):
        self.local = threading.local()
        Sqlite3Connection.__init__(self, config, False)
        self.check_same_thread = True
        if connectNow:
            self.connect()

        
class Sqlite3ConnRequest(Sqlite3Connection, ConnectionRequest):
    """
    Caches database connections as request values. Uses thread local stack as fallback (e.g testing).
    """

    def __init__(self, config = None, connectNow = True):
        self.local = threading.local()
        Sqlite3Connection.__init__(self, config, False)
        self.check_same_thread = True
        if connectNow:
            self.connect()
    



class Sqlite3(FileManager, Base):
    """
    Data Pool Sqlite3 implementation
    """
    _OperationalError = sqlite3.OperationalError
    _DefaultConnection = Sqlite3ConnRequest
    _EmptyValues = []


    def GetContainedIDs(self, base=0, sort=u"title", parameter=u""):
        """
        select list of all entries
        id needs to be first field, pool_unitref second
        """
        def _SelectIDs(base, ids, sql, cursor):
            cursor.execute(sql%(base))
            entries = cursor.fetchall()
            for e in entries:
                ids.append(e[0])
                ids = _SelectIDs(e[0], ids, sql, cursor)
            return ids
        parameter = u"where pool_unitref=%d " + parameter
        sql = u"""select id from %s %s order by %s""" % (self.MetaTable, parameter, sort)
        cursor = self.connection.cursor()
        ids = _SelectIDs(base, [], sql, cursor)
        cursor.close()
        return ids


    def GetTree(self, flds=[u"id"], sort=u"title", base=0, parameter=u""):
        """
        select list of all entries
        id needs to be first field if flds
        """
        def _Select(base, tree, flds, sql, cursor):
            cursor.execute(sql%(base))
            entries = cursor.fetchall()
            for e in entries:
                data = self.ConvertRecToDict(e, flds)
                data.update({"id": e[0], "items": []})
                tree["items"].append(data)
                data = _Select(e[0], data, flds, sql, cursor)
            return tree

        if parameter != u"":
            parameter = u"and " + parameter
        parameter = u"where pool_unitref=%d " + parameter
        sql = u"""select %s from %s %s order by %s""" % (ConvertListToStr(flds), self.MetaTable, parameter, sort)
        cursor = self.connection.cursor()
        tree = {"id":base, "items":[]}
        tree = _Select(base, tree, flds, sql, cursor)
        cursor.close()
        return tree


    def _GetInsertIDValue(self, cursor):
        cursor.execute(u"SELECT last_insert_rowid()")
        return cursor.fetchone()[0]

               
    def _CreateNewID(self, table = u"", dataTbl = None):
        #
        aC = self.connection.cursor()
        if table == "":
            table = self.MetaTable
        if table == self.MetaTable:
            if not dataTbl:
                raise "Missing data table", "Entry not created"

            # sql insert empty rec in data table
            if self._debug:
                STACKF(0,u"INSERT INTO %s (id) VALUES(null)" % (dataTbl)+"\r\n",self._debug, self._log,name=self.name)
            aC.execute(u"INSERT INTO %s (id) VALUES(null)" % (dataTbl))
            # sql get id of created rec
            if self._debug:
                STACKF(0,u"SELECT last_insert_rowid()\r\n",0, self._log,name=self.name)
            aC.execute(u"SELECT last_insert_rowid()")
            dataref = aC.fetchone()[0]
            # sql insert empty rec in meta table
            if self._debug:
                STACKF(0,u"INSERT INTO %s (pool_datatbl, pool_dataref) VALUES ('%s', %s)"% (self.MetaTable, dataTbl, dataref),0, self._log,name=self.name)
            aC.execute(u"INSERT INTO %s (pool_datatbl, pool_dataref) VALUES ('%s', %s)"% (self.MetaTable, dataTbl, dataref))
            if self._debug:
                STACKF(0,u"SELECT last_insert_rowid()\r\n",0, self._log,name=self.name)
            aC.execute(u"SELECT last_insert_rowid()")
            aID = aC.fetchone()[0]
            aC.close()
            return aID, dataref

        # sql insert empty rec in meta table
        if self._debug:
            STACKF(0,u"INSERT INTO %s (id) VALUES(null)" % (table)+"\r\n",self._debug, self._log,name=self.name)
        aC.execute(u"INSERT INTO %s (id) VALUES(null)" % (table))
        # sql get id of created rec
        if self._debug:
            STACKF(0,u"SELECT last_insert_rowid()\r\n",0, self._log,name=self.name)
        aC.execute(u"SELECT last_insert_rowid()")
        aID = aC.fetchone()[0]
        aC.close()
        return aID, 0


    def _CreateFixID(self, id, dataTbl):
        if self.IsIDUsed(id):
            raise TypeError, "ID already in use"
        aC = self.connection.cursor()
        ph = self.placeholder
        if not dataTbl:
            raise TypeError, "Missing data table."

        # sql insert empty rec in data table
        if self._debug:
            STACKF(0,u"INSERT INTO %s (id) VALUES(null)" % (dataTbl)+"\r\n",self._debug, self._log,name=self.name)
        aC.execute(u"INSERT INTO %s (id) VALUES(null)" % (dataTbl))
        # sql get id of created rec
        if self._debug:
            STACKF(0,u"SELECT last_insert_rowid()\r\n",0, self._log,name=self.name)
        aC.execute(u"SELECT last_insert_rowid()")
        dataref = aC.fetchone()[0]
        # sql insert empty rec in meta table
        if self._debug:
            STACKF(0,u"INSERT INTO %s (id, pool_datatbl, pool_dataref) VALUES (%d, '%s', %s)"% (self.MetaTable, id, dataTbl, dataref),0, self._log,name=self.name)
        aC.execute(u"INSERT INTO %s (id, pool_datatbl, pool_dataref) VALUES (%d, '%s', %s)"% (self.MetaTable, id, dataTbl, dataref))
        if self._debug:
            STACKF(0,u"SELECT last_insert_rowid()\r\n",0, self._log,name=self.name)
        aC.execute(u"SELECT last_insert_rowid()")
        aID = aC.fetchone()[0]
        aC.close()
        return id, dataref

    # types/classes -------------------------------------------------------------------

    def _FmtListForQuery(self, values):
        FmtParam = self.connection.FmtParam
        return u",".join([FmtParam(v) for v in values])

    def _GetPoolEntry(self, id, **kw):
        try:
            return Sqlite3Entry(self, id, **kw)
        except NotFound:
            return None



class Sqlite3Entry(FileEntry, Entry):
    """
    Data Pool Entry Sqlite3 implementation
    """





# -*- coding: latin-1 -*-

import copy, time
import unittest


from nive.definitions import FieldConf
from nive.utils.dataPool2.dbManager import *
from nive.utils.path import DvPath

dbpath = ":memory:"
tablename = "testtable"
tablename2 = "testtable2"
columns = ("ftext","fnumber","fdate","flist","fmselect","funit","funitlist")

SystemFlds = (
FieldConf(**{"id": "id",            "datatype": "number",   "size": 8,  "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  1000, "name": "ID",             "description": ""}),
FieldConf(**{"id": "title",         "datatype": "string",   "size": 255,"default": "",    "required": 0,     "readonly": 0, "settings": {}, "sort":  1100, "name": "Title",            "description": ""}),
FieldConf(**{"id": "pool_type",     "datatype": "list",     "size": 35, "default": "",    "required": 1,     "readonly": 1, "settings": {}, "sort":  1200, "name": "Type",              "description": ""}),
FieldConf(**{"id": "pool_category", "datatype": "list",     "size": 35, "default": "",    "required": 0,     "readonly": 0, "settings": {}, "sort":  1300, "name": "Category",          "description": ""}),
FieldConf(**{"id": "pool_filename", "datatype": "string",   "size": 255,"default": "",    "required": 0,     "readonly": 0, "settings": {}, "sort":  1400, "name": "Filename",         "description": ""}),
FieldConf(**{"id": "pool_create",   "datatype": "datetime", "size": 0,  "default": "",    "required": 1,     "readonly": 1, "settings": {}, "sort":  1500, "name": "Created",         "description": ""}),
FieldConf(**{"id": "pool_change",   "datatype": "datetime", "size": 0,  "default": "",    "required": 1,     "readonly": 1, "settings": {}, "sort":  1600, "name": "Changed",         "description": ""}),
FieldConf(**{"id": "pool_createdby","datatype": "string",   "size": 35, "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  1700, "name": "Created by",     "description": ""}),
FieldConf(**{"id": "pool_changedby","datatype": "string",   "size": 35, "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  1800, "name": "Changed by",        "description": ""}),
FieldConf(**{"id": "pool_wfp",      "datatype": "list",     "size": 35, "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  1900, "name": "Workflow Process",     "description": ""}),
FieldConf(**{"id": "pool_wfa",      "datatype": "list",     "size": 35, "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  2000, "name": "Workflow Activity",    "description": ""}),
FieldConf(**{"id": "pool_state",    "datatype": "number",   "size": 4,  "default": 1,     "required": 0,     "readonly": 1, "settings": {}, "sort":  2100, "name": "State",             "description": ""}),
FieldConf(**{"id": "pool_sort",     "datatype": "number",   "size": 8,  "default": 0,     "required": 0,     "readonly": 1, "settings": {}, "sort":  2200, "name": "Sort",             "description": ""}),
FieldConf(**{"id": "pool_unitref",  "datatype": "number",   "size": 8,  "default": "",    "required": 0,     "readonly": 1, "settings": {}, "sort":  5000, "name": "Reference",         "description": ""}),
FieldConf(**{"id": "pool_stag",     "datatype": "number",   "size": 4,  "default": 0,     "required": 0,     "readonly": 1, "settings": {}, "sort":  5100, "name": "Select Number",     "description": ""}),
FieldConf(**{"id": "pool_datatbl",  "datatype": "string",   "size": 35, "default": "",    "required": 1,     "readonly": 1, "settings": {}, "sort":  5200, "name": "Data Table Name",    "description": ""}),
)

class Sqlite3Test(unittest.TestCase):

    def setUp(self):
        p=DvPath(dbpath)
        p.Delete()
        self.db = Sqlite3Manager()
        self.db.Connect(dbpath)
        pass

    def tearDown(self):
        self.db.Close()
        p=DvPath(dbpath)
        p.Delete()
        pass

    def test_Connect(self):
        db = Sqlite3Manager()
        self.assert_(db.Connect(dbpath))
        self.assert_(db.IsDB())
        db.Close()
        self.assert_(not db.IsDB())
        

    def test_CreateTable(self):
        try:
            self.db.DeleteTable(tablename)
        except:
            pass
        self.assertFalse(self.db.IsTable(tablename))
        self.assert_(self.db.CreateTable(tablename))
        self.assert_(self.db.IsTable(tablename))
        self.assert_(self.db.DeleteTable(tablename))
        self.assertFalse(self.db.IsTable(tablename))

        self.assert_(self.db.CreateTable(tablename))
        self.assert_(self.db.IsTable(tablename))
        self.assert_(self.db.RenameTable(tablename, tablename2))
        self.assert_(self.db.DeleteTable(tablename2))
        self.assertFalse(self.db.IsTable(tablename))
        self.assertFalse(self.db.IsTable(tablename2))


    def test_CreateColumns(self):
        self._createtbl()
        self._createcols()
        self._deltable()


    def test_CreateColumns2(self):
        self._createtbl()
        self._createcols()
        #self.db.Close()
        #self.db.Connect(dbpath)
        for col in columns:
            self.assert_(self.db.IsColumn(tablename, col))
        self._deltable()


    def test_CreateColumnsStructure(self):
        for f in SystemFlds:
            self.assert_(self.db.ConvertConfToColumnOptions(f))
        self._createtbl()
        self.assert_(self.db.UpdateStructure(tablename, SystemFlds))
        for f in SystemFlds:
            self.assert_(self.db.IsColumn(tablename, f["id"]))
        self._deltable()


    
    def _createtbl(self):
        try:
            self.db.DeleteTable(tablename)
        except:
            pass
        self.assertFalse(self.db.IsTable(tablename))
        self.assert_(self.db.CreateTable(tablename))
        self.assert_(self.db.IsTable(tablename))

        
    def _createcols(self):
        for col in columns:
            self.assert_(not self.db.IsColumn(tablename, col))
            self.assert_(self.db.CreateColumn(tablename, col))
            self.assert_(self.db.IsColumn(tablename, col))
        
        
    def _deltable(self):
        self.db.DeleteTable(tablename)
        self.assertFalse(self.db.IsTable(tablename))
        
        
        
def __test():
    unittest.main()

if __name__ == '__main__':
    __test()



# -*- coding: utf-8 -*-

import unittest

from nive.definitions import *

from nive.components.extensions.path import *
from nive.security import User
from nive.cms.tests import db_app

from nive.utils import language_data


class Path(unittest.TestCase):
    
    def setUp(self):
        self.app = db_app.app_nodb()
        pass
    
    def tearDown(self):
        pass
    
    
    def test_1(self):
        p = AlternatePath()
        p.app = self.app
        
        fn = p.EscapeFilename(u"this is a filename")
        self.assert_(fn ==    u"this_is_a_filename", fn)
        
        fn = p.EscapeFilename("this is a filename")
        self.assert_(fn ==    u"this_is_a_filename", fn)
        
        fn = p.EscapeFilename(u"this is a filename")
        self.assert_(fn ==    u"this_is_a_filename", fn)
        
        fn = p.EscapeFilename(u"This Is a Filename")
        self.assert_(fn ==    u"this_is_a_filename", fn)
        
        fn = p.EscapeFilename(u"This_Is_a_Filename")
        self.assert_(fn ==    u"this_is_a_filename", fn)
        
        fn = p.EscapeFilename(u"This_Is_a_Filename/")
        self.assert_(fn ==    u"this_is_a_filename", fn)
        
        fn = p.EscapeFilename(u"This_Is_a§+#.~$%&_Filename")
        self.assert_(fn ==    u"this_is_a_filename", fn)
        
        fn = p.EscapeFilename(u"This Is a looooooooooooooooooooooooooooooooooooooooooooooong Filename")
        self.assert_(fn ==    u"this_is_a_looooooooooooooooooooooooooooooooooooooo", fn)
        
        fn = p.EscapeFilename(u"This Is a loooooooooooooooooooooooooooooooong Filename")
        self.assert_(fn ==    u"this_is_a_loooooooooooooooooooooooooooooooong", fn)
        
        fn = p.EscapeFilename(u"This Is ä Filename")
        self.assert_(fn ==    u"this_is_a_filename", fn)
        
        
    def test_special_chars(self):
        p = AlternatePath()
        p.app = self.app
        
        for lang in language_data.languages:
            conf = language_data.GetConf(lang)
            if not conf.get("special_chars"):
                continue
            
            fn = p.EscapeFilename(u"test " + conf.get("special_chars"))
            self.assert_(fn.startswith(u"test"), fn)
            
        
        

class tdbPath(unittest.TestCase):
    
    def setUp(self):
        self.app = db_app.app()
        user = self.user = User(u"test")
        user.groups.append("group:editor")
        root = self.app.root("editor")
        self.page0 = db_app.create_page(root, user)
        self.page = db_app.create_page(self.page0, user)
    
    def tearDown(self):
        user = User(u"test")
        root = self.app.root()
        root.Delete(self.page0.id, user=user)
    
    def test_automated(self):
        title = self.page.meta.title
        filename = self.page.meta.pool_filename
        self.assert_(title!=filename)
        self.assert_(filename==self.page.EscapeFilename(title), filename)

        page2 = db_app.create_page(self.page0, self.user)
        self.assert_(page2.meta.pool_filename!=filename)
        
        page3 = db_app.create_page(self.page0, self.user)
        self.assert_(page3.meta.pool_filename!=filename)
        self.assert_(page3.meta.pool_filename!=page2.meta.pool_filename)
                
        self.assert_(self.page0[filename].id==self.page.id)
        self.assert_(self.page0[filename+u".html"].id==self.page.id)
        self.assert_(self.page0[page2.meta.pool_filename].id==page2.id)
        self.assert_(self.page0[page3.meta.pool_filename].id==page3.id)
        self.assert_(self.page0[str(self.page.id)].id==self.page.id)

        self.assertRaises(KeyError, self.page0.__getitem__, u"nofilename")
        self.assertRaises(KeyError, self.page0.__getitem__, u"9999999999")
        self.assertRaises(KeyError, self.page0.__getitem__, filename+u".html.html")

        
    def test_disabled(self):
        user = User(u"test")
        self.page.data["customfilename"] = True
        self.page.Commit(user=user)
        self.page.meta["pool_filename"] = u"custom"
        self.page.Commit(user=user)
        self.assert_(self.page.meta.pool_filename==u"custom")
        self.page.meta["title"] = u"new title"
        self.page.Commit(user=user)
        self.assert_(self.page.meta.pool_filename==u"custom")
        
        



# -*- coding: utf-8 -*-

import time
import unittest

from nive.helper import FormatConfTestFailure

from nive.cms.cmsview import view
from nive.cms.cmsview import cmsroot




class TestConf(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_conf1(self):
        r=view.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")

    def test_conf2(self):
        r=cmsroot.configuration.test()
        if not r:
            return
        print FormatConfTestFailure(r)
        self.assert_(False, "Configuration Error")


if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------


from pyramid.renderers import get_renderer, render, render_to_response

from nive.i18n import _
from nive.definitions import ViewConf, ViewModuleConf, Conf
from nive.definitions import IApplication, IUser


# view module definition ------------------------------------------------------------------

#@nive_module
configuration = ViewModuleConf(
    id = "useradmin",
    name = _(u"User management"),
    static = "",
    containment = "nive.userdb.useradmin.adminroot.adminroot",
    context = "nive.userdb.useradmin.adminroot.adminroot",
    view = "nive.userdb.useradmin.view.UsermanagementView",
    templates = "nive.userdb.useradmin:",
    permission = "manage users"
)
t = configuration.templates
configuration.views = [
    # User Management Views
    ViewConf(name = "",       attr = "view",   containment=IApplication, renderer = t+"root.pt"),
    ViewConf(name = "list",   attr = "view",   containment=IApplication, renderer = t+"root.pt"),
    ViewConf(name = "add",    attr = "add",    containment=IApplication, renderer = t+"add.pt"),
    ViewConf(name = "delete", attr = "delete", containment=IApplication, renderer = t+"delete.pt"),
    ViewConf(name = "",       attr = "edit",   context = IUser, renderer = t+"edit.pt"),
]


        
    
# view and form implementation ------------------------------------------------------------------

from nive.views import BaseView, Unauthorized, Mail
from nive.forms import ObjectForm, ValidationError
from nive.userdb.root import UsernameValidator

from nive.adminview.view import AdminBasics
    

class UsermanagementView(AdminBasics):
    
    
    def GetAdminWidgets(self):
        url = self.FolderUrl(self.context.root())
        confs = [
            Conf(id="admin.root", viewmapper=url+"list", name=_(u"List users")),
            Conf(id="admin.add", viewmapper=url+"add", name=_(u"Add user"))
        ]
        return confs


    def view(self):
        return {}


    def add(self):
        name = self.context.app.GetObjectFld("name", "user").copy()
        name.settings["validator"] = UsernameValidator
        form = ObjectForm(loadFromType="user", view=self)
        form.subsets = {
            "create": {"fields":  [name, "password", "email", "groups", "surname", "lastname"], 
                       "actions": ["create"],
                       "defaultAction": "default"}
        }
        form.Setup(subset="create")
        result, data, action = form.Process(redirectSuccess="obj_url", pool_type="user")
        return {u"content": data, u"result": result, u"head": form.HTMLHead()}


    def edit(self):
        pwd = self.context.app.GetObjectFld("password", "user").copy()
        pwd.settings["update"] = True
        pwd.required = False
        form = ObjectForm(loadFromType="user", subset="edit", view=self)
        def removepasswd(data, obj):
            try:
                del data["password"]
            except:
                pass
        form.ListenEvent("loadDataObj", removepasswd)
        form.subsets = {
            "edit":   {"fields":  [pwd, "email", "groups", "surname", "lastname"], 
                       "actions": ["edit"],
                       "defaultAction": "defaultEdit"},
        }        
        form.Setup(subset="edit")
        result, data, action = form.Process()#, redirectSuccess="obj_url")
        return {u"content": data, u"result": result, u"head": form.HTMLHead()}
            
    
    def delete(self):
        ids = self.GetFormValue("ids")
        confirm = self.GetFormValue("confirm")
        users = []
        msgs = []
        root = self.context.root()
        if isinstance(ids, basestring):
            ids = (ids,)
        elif not ids:
            ids = ()
        for i in ids:
            u = root.GetUserByID(i, activeOnly=0)
            if not u:
                msgs.append(self.Translate(_(u"User not found. (id %(name)s)", mapping={"name": i})))
            else:
                users.append(u)
        result = True
        if confirm:
            for u in users:
                name = u.data.name
                if not root.Delete(id=u.id, obj=u, user=self.User()):
                    result = False
                    msgs.append(self.Translate(_(u"Delete failed: User '%(name)s'", mapping={"name": u.meta.title})))
            users=()
            if result:
                if len(ids)>1:
                    msgs.append(self.Translate(_(u"OK. Users deleted.")))
                else:
                    msgs.append(self.Translate(_(u"OK. User deleted.")))
            return self.Redirect(self.Url(root), msgs)
        return {"ids": ids, "users":users, "result": result, "msgs": msgs, "confirm": confirm} 
    










# -*- coding: latin-1 -*-

import copy
from time import time
import unittest
from types import UnicodeType

from nive.definitions import DatabaseConf
from nive.utils.dataPool2.base import *
from nive.utils.dataPool2.sqlite3Pool import Sqlite3
from nive.utils.path import DvPath

from sqlite3 import OperationalError

from nive.tests.db_app import app_db
from test_Base import conf, stdMeta, struct, SystemFlds, Fulltext, Files, data1_1, data2_1, meta1, file1_1, file1_2

from nive.tests import __local

# configuration ---------------------------------------------------------------------------
conn = DatabaseConf(
    dbName = __local.ROOT+"nive.db"
)

def getPool():
    p = Sqlite3(connParam=conn, **conf)
    p.structure.Init(structure=struct, stdMeta=struct[u"pool_meta"])
    return p


countdb = 1

class dbTest(object):

    def tearDown(self):
        self.pool.Close()

    def statdb(self):
        c = self.pool.GetCountEntries()
        countdb = c
        #print "Count entries in DB:", c
        return c

    def checkdb(self):
        app_db()


    # entries ---------------------------------------------------------------------------
    def create1(self):
        #print "Create Entry",
        e=self.pool.CreateEntry(u"data1", user="unittest")
        e.Commit(user="unittest")
        #print e.GetID(), "OK"
        return e.GetID()

    def create2(self):
        #print "Create Entry",
        e=self.pool.CreateEntry(u"data2", user="unittest")
        e.Commit(user="unittest")
        #print e.GetID(), "OK"
        return e.GetID()

    def get(self, id):
        #print "Load entry", id
        e=self.pool.GetEntry(id)
        #print "OK"
        return e

    # set ---------------------------------------
    def set1(self, id):
        #print "Store data", id,
        e=self.pool.GetEntry(id)
        self.assert_(e)
        e.data.update(data1_1)
        e.meta.update(meta1)
        e.Commit(user="unittest")
        self.assert_(e.GetMeta())
        d=e.GetData()
        self.assert_(d.get(u"ftext")    == data1_1.get(u"ftext")    )
        self.assert_(d.get(u"fnumber")    == data1_1.get(u"fnumber")    )
        self.assert_(self.pool.GetDBDate(str(d.get(u"fdate"))) == self.pool.GetDBDate(str(data1_1.get(u"fdate"))))
        self.assert_(d.get(u"flist")    == data1_1.get(u"flist")    )
        self.assert_(d.get(u"fmselect") == data1_1.get(u"fmselect") )
        self.assert_(d.get(u"funit")    == data1_1.get(u"funit")    )
        self.assert_(d.get(u"funitlist")== data1_1.get(u"funitlist"))
        self.assert_(type(d.get(u"ftext"))==UnicodeType)
        #print "OK"

    def set2(self, id):
        #print "Store data", id,
        e=self.pool.GetEntry(id)
        self.assert_(e)
        e.data.update(data2_1)
        e.meta.update(meta1)
        e.Commit(user="unittest")
        self.assert_(e.GetMeta())
        d=e.GetData()
        self.assert_(d.get(u"ftext")    == data2_1.get(u"ftext")    )
        self.assert_(d.get(u"fstr")     == data2_1.get(u"fstr")    )
        self.assert_(type(d.get(u"ftext"))==UnicodeType    )
        #print "OK"

    def setfile1(self, id):
        #print "Store file", id,
        e=self.pool.GetEntry(id)
        self.assert_(e)
        self.assert_(e.CommitFile(u"file1", {"file":file1_1, "filename":"file1.txt"}))
        e.Commit(user="unittest")
        self.assert_(e.GetFile(u"file1").read() == file1_1)
        #print "OK"

    def setfile2(self, id):
        #print "Store file", id,
        e=self.pool.GetEntry(id)
        self.assert_(e)
        self.assert_(e.CommitFile(u"file2", {"file":file1_2, "filename":u"file2.txt"}))
        e.Commit(user="unittest")
        self.assert_(e.GetFile(u"file2").read() == file1_2)
        #print "OK"


    # get --------------------------------------------------------------------------
    def data1(self, id):
        #print "Check entry data", id,
        e=self.pool.GetEntry(id)
        d=e.GetData()
        self.assert_(d.get(u"ftext")    == data1_1.get(u"ftext")    )
        self.assert_(d.get(u"fnumber")    == data1_1.get(u"fnumber")    )
        self.assert_(self.pool.GetDBDate(str(d.get(u"fdate"))) == self.pool.GetDBDate(str(data1_1.get(u"fdate"))))
        self.assert_(d.get(u"flist")    == data1_1.get(u"flist")    )
        self.assert_(d.get(u"fmselect") == data1_1.get(u"fmselect") )
        self.assert_(d.get(u"funit")    == data1_1.get(u"funit")    )
        self.assert_(d.get(u"funitlist")== data1_1.get(u"funitlist"))
        self.assert_(type(d.get(u"ftext"))==UnicodeType    )
        #print "OK"

    def data2(self, id):
        #print "Check entry data", id,
        e=self.pool.GetEntry(id)
        d=e.GetData()
        self.assert_(d.get(u"ftext")    == data2_1.get(u"ftext")    )
        self.assert_(d.get(u"fstr")     == data2_1.get(u"fstr")    )
        self.assert_(type(d.get(u"ftext"))==UnicodeType    )
        #print "OK"

    def file1(self, id):
        #print "Load file", id,
        e=self.pool.GetEntry(id)
        self.assert_(e.GetFile(u"file1").read() == file1_1)
        #print "OK"

    def file2(self, id):
        #print "Load file", id,
        e=self.pool.GetEntry(id)
        self.assert_(e.GetFile(u"file2").read() == file1_2)
        #print "OK"

    def fileErr(self, id):
        #print "Load non existing file", id,
        e=self.pool.GetEntry(id)
        self.assert_(e.GetFile(u"file1") == None)
        #print "OK"

    # getstream --------------------------------------------------------------------------
    def file1stream(self, id):
        #print "Load file", id,
        e=self.pool.GetEntry(id)
        s=e.GetFile(u"file1")
        d = s.read()
        s.close()
        self.assert_(d == file1_1)
        #print "OK"

    def file2stream(self, id):
        #print "Load file", id,
        e=self.pool.GetEntry(id)
        s=e.GetFile(u"file2")
        s.read()
        s.close()
        self.assert_(d == file1_2)
        #print "OK"

    # functions ------------------------------------------------------------------------
    def stat(self, id):
        e=self.pool.GetEntry(id)
        self.assert_(e.GetMetaField(u"pool_createdby")==u"unittest")
        self.assert_(e.GetMetaField(u"pool_changedby")==u"unittest")
        #print "Create: %s by %s    Changed: %s by %s" % (e.GetMetaField("pool_create"), e.GetMetaField("pool_createdby"), e.GetMetaField("pool_change"), e.GetMetaField("pool_changedby"))

    def delete(self, id):
        #print "Delete", id,
        e=self.pool.GetEntry(id)
        self.assert_(e)
        del e
        self.assert_(self.pool.DeleteEntry(id))
        self.pool.Commit(user="unittest")

    def duplicate(self, id,file=True):
        #print "Duplicate", id,
        e=self.pool.GetEntry(id)
        n=e.Duplicate(file)
        self.assert_(n)
        n.Commit(user="unittest")
        #print "OK"
        return n.GetID()

    def filetest(self, id):
        #print "File test", id,
        e=self.pool.GetEntry(id)
        self.assert_(e)

        self.assertItemsEqual(e.FileKeys(), [u"file1",u"file2"])

        self.assert_(e.GetFile(u"file1").filename==u"file1.txt")
        self.assert_(e.GetFile(u"file2").filename==u"file2.txt")
        self.assert_(type(e.GetFile(u"file1").filename)==UnicodeType)
        self.assert_(type(e.GetFile(u"file2").filename)==UnicodeType)

        self.assert_(e.GetFile(u"file1"))
        self.assert_(e.GetFile(u"file2"))
        self.assert_(e.GetFile(u"file3")==None)
        self.assert_(e.GetFile(u"")==None)

        l=e.Files({})
        self.assert_(len(l)==2)
        l2=[]
        for f in l:
            l2.append(f[u"filekey"])
        self.assert_(u"file1" in l2)
        self.assert_(u"file2" in l2)
        #print "OK"





    # helper ------------------------------------------------------------------------
    def bakupSQL(self):
        #print "Bakup SQL Dump to file"
        p = PoolBakup(self.pool)
        r,p1 = p.SQLDump()
        r,p2 = p.SQLArchive()
        #print r
        p1 = DvPath(p1)
        self.assert_(p1.Exists())
        p2 = DvPath(p2)
        self.assert_(p2.Exists())


    # new test fncs -----------------------------------------------------------------------------

    def test_create_empty(self):

        t = time()
        c = self.statdb()
        # creating
        id1=self.create1()
        e=self.get(id1)
        del e
        self.stat(id1)

        id2=self.create2()
        e=self.get(id2)
        del e
        self.stat(id2)

        # cnt    ok
        c2 = self.statdb()
        self.assert_(c+2==c2)

        # deleting
        self.delete(id1)
        self.delete(id2)
        c3 = self.statdb()
        self.assert_(c==c3)




    def test_create_base(self):

        t = time()
        c = self.statdb()
        # creating
        id1=self.create1()
        e=self.get(id1)
        del e
        self.stat(id1)

        id2=self.create2()
        e=self.get(id2)
        del e
        self.stat(id2)

        # cnt    ok
        c2 = self.statdb()
        self.assert_(c+2==c2)

        #update data
        self.set1(id1)
        self.set2(id2)
        self.setfile1(id1)
        self.setfile2(id1)

        #load data
        self.data1(id1)
        self.data2(id2)
        self.file1(id1)
        self.file2(id1)

        # deleting
        self.delete(id1)
        self.delete(id2)
        c3 = self.statdb()
        self.assert_(c==c3)



    def test_files_base(self):

        t = time()
        c = self.statdb()

        # creating
        id1=self.create1()
        e=self.get(id1)

        #update data
        self.set1(id1)
        self.setfile1(id1)
        self.setfile2(id1)

        #load data
        self.file1(id1)
        self.file2(id1)

        # file
        self.filetest(id1)

        # deleting
        self.delete(id1)
        c3 = self.statdb()
        self.assert_(c==c3)


    def test_preload(self):

        t = time()
        self.statdb()

        id=self.create1()

        #print "Preload Skip", id,
        e = self.pool.GetEntry(id, preload=u"skip")
        self.assert_(e.GetDataRef()>0 and e.GetDataTbl()!=u"")
        del e
        #print "OK"

        #print "Preload Meta", id,
        e = self.pool.GetEntry(id, preload=u"meta")
        self.assert_(e.GetDataRef()>0 and e.GetDataTbl()!=u"")
        del e
        #print "OK"

        #print "Preload All", id,
        e = self.pool.GetEntry(id, preload=u"all")
        self.assert_(e.GetDataRef()>0 and e.GetDataTbl()!=u"")
        del e
        #print "OK"

        #print "Preload MetaData", id,
        e = self.pool.GetEntry(id, preload=u"metadata")
        self.assert_(e.GetDataRef()>0 and e.GetDataTbl()!=u"")
        del e
        #print "OK"

        #print "Preload StdMeta", id,
        e = self.pool.GetEntry(id, preload=u"stdmeta")
        self.assert_(e.GetDataRef()>0 and e.GetDataTbl()!=u"")
        del e
        #print "OK"

        #print "Preload StdMetaData", id,
        e = self.pool.GetEntry(id, preload=u"stdmetadata")
        self.assert_(e.GetDataRef()>0 and e.GetDataTbl()!=u"")
        del e
        #print "OK"

        self.delete(id)
        self.assert_(self.pool.IsIDUsed(id) == False)


    def test_duplicate_base(self):

        t = time()
        c=self.statdb()

        # creating
        id1=self.create1()
        id2=self.create2()

        # cnt    ok
        c2 = self.statdb()
        self.assert_(c+2==c2)

        #update data
        self.set1(id1)
        self.set2(id2)
        self.setfile1(id1)
        self.setfile2(id1)

        #load data
        self.data1(id1)
        self.data2(id2)
        self.file1(id1)
        self.file2(id1)

        # duplicate
        id3=self.duplicate( id1)
        id4=self.duplicate( id2)
        id5=self.duplicate( id1, file=False)

        #load dupl. data
        self.data1(id3)
        self.data1(id5)
        self.data2(id4)
        self.file1(id3)
        self.file2(id3)
        self.fileErr(id5)

        # deleting
        self.delete(id1)
        self.delete(id2)
        self.delete(id3)
        self.delete(id4)
        self.delete(id5)
        c3 = self.statdb()
        self.assert_(c==c3)


    def test_sql(self):

        t = time()
        sql, values=self.pool.FmtSQLSelect(list(stdMeta)+list(struct[u"data1"]),
                        {u"pool_type": "data1", u"ftext": "123", u"fnumber": 300000},
                        sort = u"title, id, fnumber",
                        ascending = 0,
                        dataTable = u"data1",
                        operators={u"pool_type":u"=", u"ftext": u"<>", u"fnumber": u"<"},
                        start=1,
                        max=123)
        self.pool.Query(sql, values)
        c=self.pool.Execute(sql, values)
        c.close()
        #print "OK"

        sql, values=self.pool.FmtSQLSelect(list(struct[u"data1"]),
                                     {u"ftext": "", u"fnumber": 3},
                                     dataTable=u"data1",
                                     sort = u"id, fnumber",
                                     ascending = 1,
                                     operators={u"ftext": u"=", u"fnumber": u"="},
                                     start=1,
                                     max=123,
                                     singleTable=1)
        self.pool.Query(sql, values)
        c=self.pool.Execute(sql, values)
        c.close()
        #print "OK"

        #print "GetFulltextSQL",
        sql, values=self.pool.GetFulltextSQL(u"is",
                            list(stdMeta)+list(struct[u"data1"]),
                            {},
                            sort = u"title",
                            ascending = 1,
                            dataTable = u"data1")
        self.pool.Query(sql, values)
        c=self.pool.Execute(sql, values)
        c.close()
        #print "OK"


    def test_sql2(self):

        t = time()
        sql1, values1=self.pool.FmtSQLSelect(list(stdMeta)+list(struct[u"data1"]),
                        {u"pool_type": "data1", u"ftext": "", u"fnumber": 3},
                        sort = u"title, id, fnumber",
                        ascending = 0,
                        dataTable = u"data1",
                        operators={u"pool_type":u"=", u"ftext": u"<>", u"fnumber": u">"},
                        start=1,
                        max=123)
        sql2, values2=self.pool.FmtSQLSelect(list(struct[u"data1"]),
                                     {u"ftext": u"", u"fnumber": 3},
                                     dataTable=u"data1",
                                     sort = u"id, fnumber",
                                     ascending = 1,
                                     operators={u"ftext": u"<>", u"fnumber": u">"},
                                     start=1,
                                     max=123,
                                     singleTable=1)
        sql3, values3=self.pool.GetFulltextSQL(u"is",
                            list(stdMeta)+list(struct[u"data1"]),
                            {},
                            sort = u"title",
                            ascending = 1,
                            dataTable = u"data1")
        c=self.pool.connection.cursor()
        c.execute(sql1, values1)
        c.execute(sql2, values2)
        c.execute(sql3, values3)
        
        self.pool.SelectFields(u"data1", fields=(u"id",), idValues=[0], idColumn=u"id")
        self.pool.SelectFields(u"pool_meta", fields=(u"id",u"title",u"pool_type"), idValues=[1,2,3,4,5], idColumn=u"pool_unitref")


    def test_insertdelete(self):
        self.pool.DeleteRecords("pool_meta", {"pool_type": "notype", "title": "test entry"}, cursor=None)
        self.pool.Commit()

        self.pool.InsertFields("pool_meta", {"pool_type": "notype", "title": "test entry"}, cursor = None)
        self.pool.Commit()
        sql, values = self.pool.FmtSQLSelect(["id"], {"pool_type": "notype", "title": "test entry"}, dataTable="pool_meta", singleTable=1) 
        id = self.pool.Query(sql, values)
        self.assert_(id)
        
        self.pool.UpdateFields("pool_meta", id[0][0], {"pool_type": "notype 123", "title": "test entry 123"}, cursor = None)
        self.pool.Commit()
        sql, values = self.pool.FmtSQLSelect(["id"], {"pool_type": "notype", "title": "test entry"}, dataTable="pool_meta", singleTable=1) 
        id = self.pool.Query(sql, values)
        self.assertFalse(id)
        sql, values = self.pool.FmtSQLSelect(["id"], {"pool_type": "notype 123", "title": "test entry 123"}, dataTable="pool_meta", singleTable=1) 
        id = self.pool.Query(sql, values)
        self.assert_(id)
        
        for i in id:
            self.pool.DeleteRecords("pool_meta", {"id":i[0]}, cursor=None)
        self.pool.Commit()
        sql, values = self.pool.FmtSQLSelect(["id"], {"pool_type": "notype 123", "title": "test entry 123"}, dataTable="pool_meta", singleTable=1) 
        id = self.pool.Query(sql, values)
        self.assertFalse(id)


    def test_groups(self):
        userid = 123
        group = u"group:test"
        id = 1
        ref = u"o"
        self.pool.RemoveGroups(id=id)
        self.assertFalse(self.pool.GetGroups(id, userid, group))
        self.assertFalse(self.pool.GetGroups(id))
        self.pool.AddGroup(id, userid, group)
        self.assert_(self.pool.GetGroups(id, userid, group))
        self.assert_(self.pool.GetGroups(id))
        
        self.pool.RemoveGroups(userid=userid, group=group, id=id)
        self.assertFalse(self.pool.GetGroups(id, userid, group))




    def test_search_files(self):

        t = time()
        c = self.statdb()

        # creating
        id1=self.create1()
        id2=self.create1()
        id3=self.create1()

        #update data
        self.set1(id1)
        self.setfile1(id1)
        self.setfile2(id1)

        self.set1(id2)
        self.setfile1(id2)
        self.setfile2(id2)

        self.set1(id3)
        self.setfile1(id3)
        self.setfile2(id3)

        dbfile = self.pool
        #print "SearchFilename",
        f1 = dbfile.SearchFilename(u"file1.txt")
        #print len(f1),
        self.assert_(len(f1)>=3)
        f2 = dbfile.SearchFilename(u"file2.txt")
        #print len(f2),
        self.assert_(len(f2)>=3)
        f3 = dbfile.SearchFilename(u"fileXXX.txt")
        #print len(f3),
        self.assert_(len(f3)==0)
        f4 = dbfile.SearchFilename(u"file%")
        #print len(f4),
        if countdb == 0:
            self.assert_(len(f4)>=len(f1)+len(f2))
        #print "OK"

        #print "SearchFiles",
        parameter={u"id": (id1,id2,id3)}
        operators={u"id": u"IN"}
        f = dbfile.SearchFiles(parameter, operators=operators) #sort="filename",
        #print len(f),
        if countdb==0:
            self.assert_(len(f)==6)
        parameter[u"filename"] = u"file2.txt"
        operators[u"filename"] = u"="
        f = dbfile.SearchFiles(parameter, operators=operators) #sort="size",
        #print len(f),
        if countdb==0:
            self.assert_(len(f)==3)
        #print "OK"

        # deleting
        self.delete(id1)
        self.delete(id2)
        self.delete(id3)
        c3 = self.statdb()
        self.assert_(c==c3)


    def test_tree(self):
        base = self.pool
        #base.GetContainedIDs(base=0, sort=u"title", parameter=u"")
        #base.GetTree(flds=[u"id"], sort=u"title", base=0, parameter=u"")
        base.GetParentPath(1)
        base.GetParentTitles(1)





class Sqlite3Test(dbTest, unittest.TestCase):
    """
    """
    def setUp(self):
        self.pool = getPool()
        dbfile = DvPath(conn["dbName"])
        if not dbfile.IsFile():
            dbfile.CreateDirectories()
        self.checkdb()
        self.connect()

    def tearDown(self):
        self.pool.Close()

    def connect(self):
        #print "Connect DB on", conn["host"],
        self.pool.CreateConnection(conn)
        self.assert_(self.pool.connection.IsConnected())
        #print "OK"

    

if __name__ == '__main__':
    unittest.main()


